@@ -1,21 +1,21 @@
 /*
      This file is part of GNUnet.
-     (C) 2003--2013 Christian Grothoff (and other contributing authors)
+     Copyright (C) 2003--2014 GNUnet e.V.
 
-     GNUnet is free software; you can redistribute it and/or modify
-     it under the terms of the GNU General Public License as published
-     by the Free Software Foundation; either version 3, or (at your
-     option) any later version.
+     GNUnet is free software: you can redistribute it and/or modify it
+     under the terms of the GNU Affero General Public License as published
+     by the Free Software Foundation, either version 3 of the License,
+     or (at your option) any later version.
 
      GNUnet is distributed in the hope that it will be useful, but
      WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-     General Public License for more details.
+     Affero General Public License for more details.
+    
+     You should have received a copy of the GNU Affero General Public License
+     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-     You should have received a copy of the GNU General Public License
-     along with GNUnet; see the file COPYING.  If not, write to the
-     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-     Boston, MA 02111-1307, USA.
+     SPDX-License-Identifier: AGPL3.0-or-later
 */
 
 /**
@@ -96,8 +96,9 @@
  *
  * @param uri uri to convert to a unique key
  * @param key where to store the unique key
+ * @return #GNUNET_OK on success
  */
-void
+int
 GNUNET_FS_uri_to_key (const struct GNUNET_FS_Uri *uri,
 		      struct GNUNET_HashCode *key)
 {
@@ -105,25 +106,35 @@ GNUNET_FS_uri_to_key (const struct GNUNET_FS_Uri *uri,
   {
   case GNUNET_FS_URI_CHK:
     *key = uri->data.chk.chk.query;
-    return;
+    return GNUNET_OK;
   case GNUNET_FS_URI_SKS:
     GNUNET_CRYPTO_hash (uri->data.sks.identifier,
-                        strlen (uri->data.sks.identifier), key);
-    break;
+                        strlen (uri->data.sks.identifier),
+			key);
+    return GNUNET_OK;
   case GNUNET_FS_URI_KSK:
     if (uri->data.ksk.keywordCount > 0)
+    {
       GNUNET_CRYPTO_hash (uri->data.ksk.keywords[0],
-                          strlen (uri->data.ksk.keywords[0]), key);
+                          strlen (uri->data.ksk.keywords[0]),
+			  key);
+      return GNUNET_OK;
+    }
+    else
+    {
+      memset (key, 0, sizeof (struct GNUNET_HashCode));
+      return GNUNET_SYSERR;
+    }
     break;
   case GNUNET_FS_URI_LOC:
     GNUNET_CRYPTO_hash (&uri->data.loc.fi,
                         sizeof (struct FileIdentifier) +
                         sizeof (struct GNUNET_PeerIdentity),
                         key);
-    break;
+    return GNUNET_OK;
   default:
     memset (key, 0, sizeof (struct GNUNET_HashCode));
-    break;
+    return GNUNET_SYSERR;
   }
 }
 
@@ -200,7 +211,8 @@ GNUNET_FS_uri_ksk_to_string_fancy (const struct GNUNET_FS_Uri *uri)
  * @return decodded string with leading space (or preserved plus)
  */
 static char *
-percent_decode_keyword (const char *in, char **emsg)
+percent_decode_keyword (const char *in,
+                        char **emsg)
 {
   char *out;
   char *ret;
@@ -219,7 +231,7 @@ percent_decode_keyword (const char *in, char **emsg)
       {
         GNUNET_free (out);
         *emsg = GNUNET_strdup (_(/* xgettext:no-c-format */
-				 "`%' must be followed by HEX number"));
+				 "Malformed KSK URI (`%' must be followed by HEX number)"));
         return NULL;
       }
       rpos += 3;
@@ -258,7 +270,8 @@ percent_decode_keyword (const char *in, char **emsg)
  * @return NULL on error, otherwise the KSK URI
  */
 static struct GNUNET_FS_Uri *
-uri_ksk_parse (const char *s, char **emsg)
+uri_ksk_parse (const char *s,
+               char **emsg)
 {
   struct GNUNET_FS_Uri *ret;
   char **keywords;
@@ -270,7 +283,6 @@ uri_ksk_parse (const char *s, char **emsg)
   char *dup;
   int saw_quote;
 
-  GNUNET_assert (NULL != s);
   slen = strlen (s);
   pos = strlen (GNUNET_FS_URI_KSK_PREFIX);
   if ((slen <= pos) || (0 != strncmp (s, GNUNET_FS_URI_KSK_PREFIX, pos)))
@@ -296,25 +308,25 @@ uri_ksk_parse (const char *s, char **emsg)
       max++;
       if (s[i - 1] == '+')
       {
-        *emsg = GNUNET_strdup (_("`++' not allowed in KSK URI"));
+        *emsg = GNUNET_strdup (_("Malformed KSK URI (`++' not allowed)"));
         return NULL;
       }
     }
   }
   if (saw_quote == 1)
   {
-    *emsg = GNUNET_strdup (_("Quotes not balanced in KSK URI"));
+    *emsg = GNUNET_strdup (_("Malformed KSK URI (quotes not balanced)"));
     return NULL;
   }
   iret = max;
   dup = GNUNET_strdup (s);
-  keywords = GNUNET_malloc (max * sizeof (char *));
+  keywords = GNUNET_new_array (max,
+                               char *);
   for (i = slen - 1; i >= (int) pos; i--)
   {
     if ((s[i] == '%') && (&s[i] == strstr (&s[i], "%22")))
     {
       saw_quote = (saw_quote + 1) % 2;
-      i += 3;
       continue;
     }
     if ((dup[i] == '+') && (saw_quote == 0))
@@ -328,7 +340,7 @@ uri_ksk_parse (const char *s, char **emsg)
   keywords[--max] = percent_decode_keyword (&dup[pos], emsg);
   if (NULL == keywords[max])
     goto CLEANUP;
-  GNUNET_assert (max == 0);
+  GNUNET_assert (0 == max);
   GNUNET_free (dup);
   ret = GNUNET_new (struct GNUNET_FS_Uri);
   ret->type = GNUNET_FS_URI_KSK;
@@ -354,14 +366,14 @@ CLEANUP:
  * @return NULL on error, SKS URI otherwise
  */
 static struct GNUNET_FS_Uri *
-uri_sks_parse (const char *s, char **emsg)
+uri_sks_parse (const char *s,
+               char **emsg)
 {
   struct GNUNET_FS_Uri *ret;
   struct GNUNET_CRYPTO_EcdsaPublicKey ns;
   size_t pos;
   char *end;
 
-  GNUNET_assert (s != NULL);
   pos = strlen (GNUNET_FS_URI_SKS_PREFIX);
   if ((strlen (s) <= pos) || (0 != strncmp (s, GNUNET_FS_URI_SKS_PREFIX, pos)))
     return NULL;                /* not an SKS URI */
@@ -373,7 +385,7 @@ uri_sks_parse (const char *s, char **emsg)
 				       &ns,
 				       sizeof (ns))) )
   {
-    *emsg = GNUNET_strdup (_("Malformed SKS URI"));
+    *emsg = GNUNET_strdup (_("Malformed SKS URI (wrong syntax)"));
     return NULL; /* malformed */
   }
   end++; /* skip over '/' */
@@ -395,7 +407,8 @@ uri_sks_parse (const char *s, char **emsg)
  * @return NULL on error, CHK URI otherwise
  */
 static struct GNUNET_FS_Uri *
-uri_chk_parse (const char *s, char **emsg)
+uri_chk_parse (const char *s,
+               char **emsg)
 {
   struct GNUNET_FS_Uri *ret;
   struct FileIdentifier fi;
@@ -405,9 +418,6 @@ uri_chk_parse (const char *s, char **emsg)
   char h1[sizeof (struct GNUNET_CRYPTO_HashAsciiEncoded)];
   char h2[sizeof (struct GNUNET_CRYPTO_HashAsciiEncoded)];
 
-  if (NULL == s)
-    return NULL;
-  GNUNET_assert (s != NULL);
   slen = strlen (s);
   pos = strlen (GNUNET_FS_URI_CHK_PREFIX);
   if ((slen < pos + 2 * sizeof (struct GNUNET_CRYPTO_HashAsciiEncoded) + 1) ||
@@ -416,12 +426,12 @@ uri_chk_parse (const char *s, char **emsg)
   if ((s[pos + sizeof (struct GNUNET_CRYPTO_HashAsciiEncoded) - 1] != '.') ||
       (s[pos + sizeof (struct GNUNET_CRYPTO_HashAsciiEncoded) * 2 - 1] != '.'))
   {
-    *emsg = GNUNET_strdup (_("Malformed CHK URI"));
+    *emsg = GNUNET_strdup (_("Malformed CHK URI (wrong syntax)"));
     return NULL;
   }
-  memcpy (h1, &s[pos], sizeof (struct GNUNET_CRYPTO_HashAsciiEncoded));
+  GNUNET_memcpy (h1, &s[pos], sizeof (struct GNUNET_CRYPTO_HashAsciiEncoded));
   h1[sizeof (struct GNUNET_CRYPTO_HashAsciiEncoded) - 1] = '\0';
-  memcpy (h2, &s[pos + sizeof (struct GNUNET_CRYPTO_HashAsciiEncoded)],
+  GNUNET_memcpy (h2, &s[pos + sizeof (struct GNUNET_CRYPTO_HashAsciiEncoded)],
           sizeof (struct GNUNET_CRYPTO_HashAsciiEncoded));
   h2[sizeof (struct GNUNET_CRYPTO_HashAsciiEncoded) - 1] = '\0';
 
@@ -431,7 +441,7 @@ uri_chk_parse (const char *s, char **emsg)
        SSCANF (&s[pos + sizeof (struct GNUNET_CRYPTO_HashAsciiEncoded) * 2],
                "%llu", &flen)))
   {
-    *emsg = GNUNET_strdup (_("Malformed CHK URI"));
+    *emsg = GNUNET_strdup (_("Malformed CHK URI (failed to decode CHK)"));
     return NULL;
   }
   fi.file_length = GNUNET_htonll (flen);
@@ -487,7 +497,8 @@ GNUNET_NETWORK_STRUCT_END
  * @return NULL on error, valid LOC URI otherwise
  */
 static struct GNUNET_FS_Uri *
-uri_loc_parse (const char *s, char **emsg)
+uri_loc_parse (const char *s,
+               char **emsg)
 {
   struct GNUNET_FS_Uri *uri;
   char h1[sizeof (struct GNUNET_CRYPTO_HashAsciiEncoded)];
@@ -505,16 +516,16 @@ uri_loc_parse (const char *s, char **emsg)
   pos = strlen (GNUNET_FS_URI_LOC_PREFIX);
   if ((slen < pos + 2 * sizeof (struct GNUNET_CRYPTO_HashAsciiEncoded) + 1) ||
       (0 != strncmp (s, GNUNET_FS_URI_LOC_PREFIX, pos)))
-    return NULL;                /* not an SKS URI */
+    return NULL;                /* not a LOC URI */
   if ((s[pos + sizeof (struct GNUNET_CRYPTO_HashAsciiEncoded) - 1] != '.') ||
       (s[pos + sizeof (struct GNUNET_CRYPTO_HashAsciiEncoded) * 2 - 1] != '.'))
   {
-    *emsg = GNUNET_strdup (_("SKS URI malformed"));
+    *emsg = GNUNET_strdup (_("LOC URI malformed (wrong syntax)"));
     return NULL;
   }
-  memcpy (h1, &s[pos], sizeof (struct GNUNET_CRYPTO_HashAsciiEncoded));
+  GNUNET_memcpy (h1, &s[pos], sizeof (struct GNUNET_CRYPTO_HashAsciiEncoded));
   h1[sizeof (struct GNUNET_CRYPTO_HashAsciiEncoded) - 1] = '\0';
-  memcpy (h2, &s[pos + sizeof (struct GNUNET_CRYPTO_HashAsciiEncoded)],
+  GNUNET_memcpy (h2, &s[pos + sizeof (struct GNUNET_CRYPTO_HashAsciiEncoded)],
           sizeof (struct GNUNET_CRYPTO_HashAsciiEncoded));
   h2[sizeof (struct GNUNET_CRYPTO_HashAsciiEncoded) - 1] = '\0';
 
@@ -524,7 +535,7 @@ uri_loc_parse (const char *s, char **emsg)
        SSCANF (&s[pos + sizeof (struct GNUNET_CRYPTO_HashAsciiEncoded) * 2],
                "%llu", &flen)))
   {
-    *emsg = GNUNET_strdup (_("LOC URI malformed"));
+    *emsg = GNUNET_strdup (_("LOC URI malformed (no CHK)"));
     return NULL;
   }
   ass.fi.file_length = GNUNET_htonll (flen);
@@ -534,7 +545,7 @@ uri_loc_parse (const char *s, char **emsg)
     npos++;
   if (s[npos] == '\0')
   {
-    *emsg = GNUNET_strdup (_("LOC URI malformed"));
+    *emsg = GNUNET_strdup (_("LOC URI malformed (missing LOC)"));
     goto ERR;
   }
   npos++;
@@ -542,7 +553,7 @@ uri_loc_parse (const char *s, char **emsg)
        ('.' != s[npos+GNUNET_CRYPTO_PKEY_ASCII_LENGTH]) )
   {
     *emsg =
-      GNUNET_strdup (_("LOC URI malformed (could not decode public key)"));
+      GNUNET_strdup (_("LOC URI malformed (wrong syntax for public key)"));
   }
   if (GNUNET_OK !=
       GNUNET_CRYPTO_eddsa_public_key_from_string (&s[npos],
@@ -556,13 +567,13 @@ uri_loc_parse (const char *s, char **emsg)
   npos += GNUNET_CRYPTO_PKEY_ASCII_LENGTH;
   if (s[npos++] != '.')
   {
-    *emsg = GNUNET_strdup (_("SKS URI malformed (could not find signature)"));
+    *emsg = GNUNET_strdup (_("LOC URI malformed (could not find signature)"));
     goto ERR;
   }
   if ( (strlen (&s[npos]) <= SIGNATURE_ASCII_LENGTH + 1) ||
        ('.' != s[npos + SIGNATURE_ASCII_LENGTH]) )
   {
-    *emsg = GNUNET_strdup (_("SKS URI malformed (could not decode signature)"));
+    *emsg = GNUNET_strdup (_("LOC URI malformed (wrong syntax for signature)"));
     goto ERR;
   }
   if (GNUNET_OK !=
@@ -571,19 +582,19 @@ uri_loc_parse (const char *s, char **emsg)
                                      &sig,
                                      sizeof (struct GNUNET_CRYPTO_EddsaSignature)))
   {
-    *emsg = GNUNET_strdup (_("SKS URI malformed (could not decode signature)"));
+    *emsg = GNUNET_strdup (_("LOC URI malformed (could not decode signature)"));
     goto ERR;
   }
   npos += SIGNATURE_ASCII_LENGTH;
   if (s[npos++] != '.')
   {
-    *emsg = GNUNET_strdup (_("SKS URI malformed"));
+    *emsg = GNUNET_strdup (_("LOC URI malformed (wrong syntax for expiration time)"));
     goto ERR;
   }
   if (1 != SSCANF (&s[npos], "%llu", &exptime))
   {
     *emsg =
-        GNUNET_strdup (_("SKS URI malformed (could not parse expiration time)"));
+        GNUNET_strdup (_("LOC URI malformed (could not parse expiration time)"));
     goto ERR;
   }
   ass.purpose.size = htonl (sizeof (struct LocUriAssembly));
@@ -595,7 +606,7 @@ uri_loc_parse (const char *s, char **emsg)
                                   &ass.purpose, &sig, &ass.peer.public_key))
   {
     *emsg =
-        GNUNET_strdup (_("SKS URI malformed (signature failed validation)"));
+        GNUNET_strdup (_("LOC URI malformed (signature failed validation)"));
     goto ERR;
   }
   uri = GNUNET_new (struct GNUNET_FS_Uri);
@@ -619,11 +630,19 @@ ERR:
  * @return NULL on error
  */
 struct GNUNET_FS_Uri *
-GNUNET_FS_uri_parse (const char *uri, char **emsg)
+GNUNET_FS_uri_parse (const char *uri,
+                     char **emsg)
 {
   struct GNUNET_FS_Uri *ret;
   char *msg;
 
+  if (NULL == uri)
+  {
+    GNUNET_break (0);
+    if (NULL != emsg)
+      *emsg = GNUNET_strdup (_("invalid argument"));
+    return NULL;
+  }
   if (NULL == emsg)
     emsg = &msg;
   *emsg = NULL;
@@ -832,59 +851,46 @@ GNUNET_FS_uri_loc_get_uri (const struct GNUNET_FS_Uri *uri)
 
 /**
  * Construct a location URI (this peer will be used for the location).
+ * This function should only be called from within gnunet-service-fs,
+ * as it requires the peer's private key which is generally unavailable
+ * to processes directly under the user's control.  However, for
+ * testing and as it logically fits under URIs, it is in this API.
  *
- * @param baseUri content offered by the sender
- * @param cfg configuration information (used to find our hostkey)
+ * @param base_uri content offered by the sender
+ * @param sign_key private key of the peer
  * @param expiration_time how long will the content be offered?
  * @return the location URI, NULL on error
  */
 struct GNUNET_FS_Uri *
-GNUNET_FS_uri_loc_create (const struct GNUNET_FS_Uri *baseUri,
-                          const struct GNUNET_CONFIGURATION_Handle *cfg,
+GNUNET_FS_uri_loc_create (const struct GNUNET_FS_Uri *base_uri,
+                          const struct GNUNET_CRYPTO_EddsaPrivateKey *sign_key,
                           struct GNUNET_TIME_Absolute expiration_time)
 {
   struct GNUNET_FS_Uri *uri;
-  struct GNUNET_CRYPTO_EddsaPrivateKey *my_private_key;
   struct GNUNET_CRYPTO_EddsaPublicKey my_public_key;
-  char *keyfile;
   struct LocUriAssembly ass;
   struct GNUNET_TIME_Absolute et;
 
-  if (baseUri->type != GNUNET_FS_URI_CHK)
+  if (GNUNET_FS_URI_CHK != base_uri->type)
     return NULL;
-  if (GNUNET_OK !=
-      GNUNET_CONFIGURATION_get_value_filename (cfg, "PEER", "PRIVATE_KEY",
-                                               &keyfile))
-  {
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                _("Lacking key configuration settings.\n"));
-    return NULL;
-  }
-  if (NULL == (my_private_key = GNUNET_CRYPTO_eddsa_key_create_from_file (keyfile)))
-  {
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                _("Could not access hostkey file `%s'.\n"), keyfile);
-    GNUNET_free (keyfile);
-    return NULL;
-  }
-  GNUNET_free (keyfile);
   /* we round expiration time to full seconds for SKS URIs */
   et.abs_value_us = (expiration_time.abs_value_us / 1000000LL) * 1000000LL;
-  GNUNET_CRYPTO_eddsa_key_get_public (my_private_key, &my_public_key);
+  GNUNET_CRYPTO_eddsa_key_get_public (sign_key,
+                                      &my_public_key);
   ass.purpose.size = htonl (sizeof (struct LocUriAssembly));
   ass.purpose.purpose = htonl (GNUNET_SIGNATURE_PURPOSE_PEER_PLACEMENT);
   ass.exptime = GNUNET_TIME_absolute_hton (et);
-  ass.fi = baseUri->data.chk;
+  ass.fi = base_uri->data.chk;
   ass.peer.public_key = my_public_key;
   uri = GNUNET_new (struct GNUNET_FS_Uri);
   uri->type = GNUNET_FS_URI_LOC;
-  uri->data.loc.fi = baseUri->data.chk;
+  uri->data.loc.fi = base_uri->data.chk;
   uri->data.loc.expirationTime = et;
   uri->data.loc.peer.public_key = my_public_key;
   GNUNET_assert (GNUNET_OK ==
-                 GNUNET_CRYPTO_eddsa_sign (my_private_key, &ass.purpose,
+                 GNUNET_CRYPTO_eddsa_sign (sign_key,
+                                           &ass.purpose,
                                            &uri->data.loc.contentSignature));
-  GNUNET_free (my_private_key);
   return uri;
 }
 
@@ -943,7 +949,8 @@ GNUNET_FS_uri_ksk_merge (const struct GNUNET_FS_Uri *u1,
     return NULL;
   }
   kc = u1->data.ksk.keywordCount;
-  kl = GNUNET_malloc ((kc + u2->data.ksk.keywordCount) * sizeof (char *));
+  kl = GNUNET_new_array (kc + u2->data.ksk.keywordCount,
+                         char *);
   for (i = 0; i < u1->data.ksk.keywordCount; i++)
     kl[i] = GNUNET_strdup (u1->data.ksk.keywords[i]);
   for (i = 0; i < u2->data.ksk.keywordCount; i++)
@@ -984,7 +991,7 @@ GNUNET_FS_uri_dup (const struct GNUNET_FS_Uri *uri)
   if (uri == NULL)
     return NULL;
   ret = GNUNET_new (struct GNUNET_FS_Uri);
-  memcpy (ret, uri, sizeof (struct GNUNET_FS_Uri));
+  GNUNET_memcpy (ret, uri, sizeof (struct GNUNET_FS_Uri));
   switch (ret->type)
   {
   case GNUNET_FS_URI_KSK:
@@ -997,8 +1004,9 @@ GNUNET_FS_uri_dup (const struct GNUNET_FS_Uri *uri)
     }
     if (ret->data.ksk.keywordCount > 0)
     {
-      ret->data.ksk.keywords =
-          GNUNET_malloc (ret->data.ksk.keywordCount * sizeof (char *));
+      ret->data.ksk.keywords
+        = GNUNET_new_array (ret->data.ksk.keywordCount,
+                            char *);
       for (i = 0; i < ret->data.ksk.keywordCount; i++)
         ret->data.ksk.keywords[i] = GNUNET_strdup (uri->data.ksk.keywords[i]);
     }
@@ -1035,7 +1043,8 @@ GNUNET_FS_uri_dup (const struct GNUNET_FS_Uri *uri)
  *  if keywords is not legal (i.e. empty).
  */
 struct GNUNET_FS_Uri *
-GNUNET_FS_uri_ksk_create (const char *keywords, char **emsg)
+GNUNET_FS_uri_ksk_create (const char *keywords,
+                          char **emsg)
 {
   char **keywordarr;
   unsigned int num_Words;
@@ -1083,7 +1092,8 @@ GNUNET_FS_uri_ksk_create (const char *keywords, char **emsg)
     *emsg = GNUNET_strdup (_("Number of double-quotes not balanced!\n"));
     return NULL;
   }
-  keywordarr = GNUNET_malloc (num_Words * sizeof (char *));
+  keywordarr = GNUNET_new_array (num_Words,
+                                 char *);
   num_Words = 0;
   inWord = 0;
   pos = searchString;
@@ -1131,7 +1141,8 @@ GNUNET_FS_uri_ksk_create (const char *keywords, char **emsg)
  *  if keywords is not legal (i.e. empty).
  */
 struct GNUNET_FS_Uri *
-GNUNET_FS_uri_ksk_create_from_args (unsigned int argc, const char **argv)
+GNUNET_FS_uri_ksk_create_from_args (unsigned int argc,
+                                    const char **argv)
 {
   unsigned int i;
   struct GNUNET_FS_Uri *uri;
@@ -1155,7 +1166,8 @@ GNUNET_FS_uri_ksk_create_from_args (unsigned int argc, const char **argv)
   uri = GNUNET_new (struct GNUNET_FS_Uri);
   uri->type = GNUNET_FS_URI_KSK;
   uri->data.ksk.keywordCount = argc;
-  uri->data.ksk.keywords = GNUNET_malloc (argc * sizeof (char *));
+  uri->data.ksk.keywords = GNUNET_new_array (argc,
+                                             char *);
   for (i = 0; i < argc; i++)
   {
     keyword = argv[i];
@@ -1184,7 +1196,7 @@ GNUNET_FS_uri_ksk_create_from_args (unsigned int argc, const char **argv)
  *
  * @param u1 one of the URIs
  * @param u2 the other URI
- * @return GNUNET_YES if the URIs are equal
+ * @return #GNUNET_YES if the URIs are equal
  */
 int
 GNUNET_FS_uri_test_equal (const struct GNUNET_FS_Uri *u1,
@@ -1250,7 +1262,7 @@ GNUNET_FS_uri_test_equal (const struct GNUNET_FS_Uri *u1,
  * Is this a namespace URI?
  *
  * @param uri the uri to check
- * @return GNUNET_YES if this is an SKS uri
+ * @return #GNUNET_YES if this is an SKS uri
  */
 int
 GNUNET_FS_uri_test_sks (const struct GNUNET_FS_Uri *uri)
@@ -1265,7 +1277,7 @@ GNUNET_FS_uri_test_sks (const struct GNUNET_FS_Uri *uri)
  *
  * @param uri the uri to get the namespace ID from
  * @param pseudonym where to store the ID of the namespace
- * @return GNUNET_OK on success
+ * @return #GNUNET_OK on success
  */
 int
 GNUNET_FS_uri_sks_get_namespace (const struct GNUNET_FS_Uri *uri,
@@ -1325,7 +1337,7 @@ GNUNET_FS_uri_test_ksk (const struct GNUNET_FS_Uri *uri)
  * Is this a file (or directory) URI?
  *
  * @param uri the uri to check
- * @return GNUNET_YES if this is a CHK uri
+ * @return #GNUNET_YES if this is a CHK uri
  */
 int
 GNUNET_FS_uri_test_chk (const struct GNUNET_FS_Uri *uri)
@@ -1361,7 +1373,7 @@ GNUNET_FS_uri_chk_get_file_size (const struct GNUNET_FS_Uri * uri)
  * Is this a location URI?
  *
  * @param uri the uri to check
- * @return GNUNET_YES if this is a LOC uri
+ * @return #GNUNET_YES if this is a LOC uri
  */
 int
 GNUNET_FS_uri_test_loc (const struct GNUNET_FS_Uri *uri)
@@ -1380,7 +1392,9 @@ GNUNET_FS_uri_test_loc (const struct GNUNET_FS_Uri *uri)
  * @param index offset where to add the keyword
  */
 static void
-insert_non_mandatory_keyword (const char *s, char **array, int index)
+insert_non_mandatory_keyword (const char *s,
+                              char **array,
+                              int index)
 {
   char *nkword;
 
@@ -1401,7 +1415,9 @@ insert_non_mandatory_keyword (const char *s, char **array, int index)
  * @return #GNUNET_YES if the keyword exists, #GNUNET_NO if not
  */
 static int
-find_duplicate (const char *s, const char **array, int array_length)
+find_duplicate (const char *s,
+                const char **array,
+                int array_length)
 {
   int j;
 
@@ -1450,7 +1466,7 @@ normalize_metadata (enum EXTRACTOR_MetaFormat format,
   {
     /* u8_tolower allocates a non-NULL-terminated string! */
     free_str = GNUNET_malloc (r_len + 1);
-    memcpy (free_str, normalized, r_len);
+    GNUNET_memcpy (free_str, normalized, r_len);
     free_str[r_len] = '\0';
     free (normalized);
     normalized = free_str;
@@ -1458,6 +1474,7 @@ normalize_metadata (enum EXTRACTOR_MetaFormat format,
   return (char *) normalized;
 }
 
+
 /**
  * Counts the number of UTF-8 characters (not bytes) in the string,
  * returns that count.
@@ -1486,14 +1503,16 @@ u8_strcount (const uint8_t *s)
  * @param array array to fill with enclosed tokens. If NULL, then tokens
  *        are only counted.
  * @param index index at which to start filling the array (entries prior
- *        to it are used to check for duplicates). ignored if array == NULL.
+ *        to it are used to check for duplicates). ignored if @a array == NULL.
  * @return number of tokens counted (including duplicates), or number of
  *         tokens extracted (excluding duplicates). 0 if there are no
  *         matching parens in the string (when counting), or when all tokens
  *         were duplicates (when extracting).
  */
 static int
-get_keywords_from_parens (const char *s, char **array, int index)
+get_keywords_from_parens (const char *s,
+                          char **array,
+                          int index)
 {
   int count = 0;
   char *open_paren;
@@ -1586,14 +1605,16 @@ get_keywords_from_parens (const char *s, char **array, int index)
  * @param array array to fill with tokens. If NULL, then tokens are only
  *        counted.
  * @param index index at which to start filling the array (entries prior
- *        to it are used to check for duplicates). ignored if array == NULL.
+ *        to it are used to check for duplicates). ignored if @a array == NULL.
  * @return number of tokens (>1) counted (including duplicates), or number of
  *         tokens extracted (excluding duplicates). 0 if there are no
  *         separators in the string (when counting), or when all tokens were
  *         duplicates (when extracting).
  */
 static int
-get_keywords_from_tokens (const char *s, char **array, int index)
+get_keywords_from_tokens (const char *s,
+                          char **array,
+                          int index)
 {
   char *p;
   char *ss;
@@ -1761,8 +1782,9 @@ GNUNET_FS_uri_ksk_create_from_meta_data (const struct GNUNET_CONTAINER_MetaData
     }
     /* x3 because there might be a normalized variant of every keyword,
        plus theoretically one more for mime... */
-    ret->data.ksk.keywords = GNUNET_malloc
-      (sizeof (char *) * (ent + tok_keywords + paren_keywords) * 3);
+    ret->data.ksk.keywords
+      = GNUNET_new_array ((ent + tok_keywords + paren_keywords) * 3,
+                          char *);
     GNUNET_CONTAINER_meta_data_iterate (md, &gather_uri_data, ret);
   }
   if (tok_keywords > 0)
