@@ -1,21 +1,21 @@
 /*
      This file is part of GNUnet.
-     (C) 2007, 2008, 2009 Christian Grothoff (and other contributing authors)
+     Copyright (C) 2007, 2008, 2009, 2014 GNUnet e.V.
 
-     GNUnet is free software; you can redistribute it and/or modify
-     it under the terms of the GNU General Public License as published
-     by the Free Software Foundation; either version 3, or (at your
-     option) any later version.
+     GNUnet is free software: you can redistribute it and/or modify it
+     under the terms of the GNU Affero General Public License as published
+     by the Free Software Foundation, either version 3 of the License,
+     or (at your option) any later version.
 
      GNUnet is distributed in the hope that it will be useful, but
      WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-     General Public License for more details.
+     Affero General Public License for more details.
+    
+     You should have received a copy of the GNU Affero General Public License
+     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-     You should have received a copy of the GNU General Public License
-     along with GNUnet; see the file COPYING.  If not, write to the
-     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-     Boston, MA 02111-1307, USA.
+     SPDX-License-Identifier: AGPL3.0-or-later
 */
 
 /**
@@ -23,10 +23,8 @@
  * @brief code for bootstrapping via hostlist servers
  * @author Christian Grothoff
  */
-
-#include <stdlib.h>
 #include "platform.h"
-#include "hostlist-client.h"
+#include "gnunet-daemon-hostlist_client.h"
 #include "gnunet_core_service.h"
 #include "gnunet_util_lib.h"
 #include "gnunet_protocols.h"
@@ -34,7 +32,7 @@
 
 #if HAVE_MHD
 
-#include "hostlist-server.h"
+#include "gnunet-daemon-hostlist_server.h"
 
 /**
  * Set if we are allowed to advertise our hostlist to others.
@@ -51,11 +49,6 @@ static int provide_hostlist;
  */
 static GNUNET_CORE_ConnectEventHandler server_ch;
 
-/**
- * Handle to hostlist server's disconnect handler
- */
-static GNUNET_CORE_DisconnectEventHandler server_dh;
-
 #endif
 
 /**
@@ -83,7 +76,7 @@ static struct GNUNET_CORE_Handle *core;
 /**
  * Handle to the hostlist client's advertisement handler
  */
-static GNUNET_CORE_MessageCallback client_adv_handler;
+static GNUNET_HOSTLIST_UriHandler client_adv_handler;
 
 /**
  * Handle to hostlist client's connect handler
@@ -123,9 +116,19 @@ struct GNUNET_HOSTLIST_ADV_Message
 };
 GNUNET_NETWORK_STRUCT_END
 
+
+/**
+ * Our own peer identity.
+ */
 static struct GNUNET_PeerIdentity me;
 
 
+/**
+ * Callback invoked once our connection to CORE service is up.
+ *
+ * @param cls NULL
+ * @param my_identity our peer's identity
+ */
 static void
 core_init (void *cls,
            const struct GNUNET_PeerIdentity *my_identity)
@@ -138,16 +141,49 @@ core_init (void *cls,
  * Core handler for p2p hostlist advertisements
  *
  * @param cls closure
- * @param peer identity of the sender
+ * @param message advertisement message we got
+ * @return #GNUNET_OK if message is well-formed
+ */
+static int
+check_advertisement (void *cls,
+		     const struct GNUNET_MessageHeader *message)
+{
+  size_t size;
+  size_t uri_size;
+  const char *uri;
+
+  size = ntohs (message->size);
+  if (size <= sizeof (struct GNUNET_MessageHeader))
+  {
+    GNUNET_break_op (0);
+    return GNUNET_SYSERR;
+  }
+  uri = (const char *) &message[1];
+  uri_size = size - sizeof (struct GNUNET_MessageHeader);
+  if (uri[uri_size - 1] != '\0')
+  {
+    GNUNET_break_op (0);
+    return GNUNET_SYSERR;
+  }
+  return GNUNET_OK;
+}
+
+
+/**
+ * Core handler for p2p hostlist advertisements
+ *
+ * @param cls closure
  * @param message advertisement message we got
  * @return #GNUNET_OK on success
  */
-static int
-advertisement_handler (void *cls, const struct GNUNET_PeerIdentity *peer,
-                       const struct GNUNET_MessageHeader *message)
+static void
+handle_advertisement (void *cls,
+		      const struct GNUNET_MessageHeader *message)
 {
+  const char *uri = (const char *) &message[1];
+ 
   GNUNET_assert (NULL != client_adv_handler);
-  return (*client_adv_handler) (cls, peer, message);
+  (void) (*client_adv_handler) (uri);
 }
 
 
@@ -157,20 +193,33 @@ advertisement_handler (void *cls, const struct GNUNET_PeerIdentity *peer,
  *
  * @param cls closure
  * @param peer peer identity this notification is about
+ * @param mq queue for sending messages to @a peer
+ * @return peer
  */
-static void
-connect_handler (void *cls, const struct GNUNET_PeerIdentity *peer)
+static void *
+connect_handler (void *cls,
+		 const struct GNUNET_PeerIdentity *peer,
+		 struct GNUNET_MQ_Handle *mq)
 {
-  if (0 == memcmp (&me, peer, sizeof (struct GNUNET_PeerIdentity)))
-    return;
+  if (0 == memcmp (&me,
+		   peer,
+		   sizeof (struct GNUNET_PeerIdentity)))
+    return NULL;
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "A new peer connected, notifying client and server\n");
   if (NULL != client_ch)
-    (*client_ch) (cls, peer);
+    GNUNET_assert (NULL ==
+		   (*client_ch) (cls,
+				 peer,
+				 mq));
 #if HAVE_MHD
   if (NULL != server_ch)
-    (*server_ch) (cls, peer);
+    GNUNET_assert (NULL ==
+		   (*server_ch) (cls,
+				 peer,
+				 mq));
 #endif
+  return (void *) peer;
 }
 
 
@@ -182,31 +231,34 @@ connect_handler (void *cls, const struct GNUNET_PeerIdentity *peer)
  * @param peer peer identity this notification is about
  */
 static void
-disconnect_handler (void *cls, const struct GNUNET_PeerIdentity *peer)
+disconnect_handler (void *cls,
+                    const struct GNUNET_PeerIdentity *peer,
+		    void *internal_cls)
 {
-  if (0 == memcmp (&me, peer, sizeof (struct GNUNET_PeerIdentity)))
+  if (0 == memcmp (&me,
+		   peer,
+		   sizeof (struct GNUNET_PeerIdentity)))
     return;
   /* call hostlist client disconnect handler */
   if (NULL != client_dh)
-    (*client_dh) (cls, peer);
-#if HAVE_MHD
-  /* call hostlist server disconnect handler */
-  if (NULL != server_dh)
-    (*server_dh) (cls, peer);
-#endif
+    (*client_dh) (cls,
+		  peer,
+		  NULL);
 }
 
 
 /**
  * Last task run during shutdown.  Disconnects us from
  * the other services.
+ *
+ * @param cls NULL
  */
 static void
-cleaning_task (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
+cleaning_task (void *cls)
 {
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
 	      "Hostlist daemon is shutting down\n");
-  if (core != NULL)
+  if (NULL != core)
   {
     GNUNET_CORE_disconnect (core);
     core = NULL;
@@ -221,9 +273,10 @@ cleaning_task (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
     GNUNET_HOSTLIST_server_stop ();
   }
 #endif
-  if (stats != NULL)
+  if (NULL != stats)
   {
-    GNUNET_STATISTICS_destroy (stats, GNUNET_NO);
+    GNUNET_STATISTICS_destroy (stats,
+			       GNUNET_NO);
     stats = NULL;
   }
 }
@@ -238,49 +291,63 @@ cleaning_task (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
  * @param cfg configuration
  */
 static void
-run (void *cls, char *const *args, const char *cfgfile,
+run (void *cls,
+     char *const *args,
+     const char *cfgfile,
      const struct GNUNET_CONFIGURATION_Handle *cfg)
 {
-  static const struct GNUNET_CORE_MessageHandler learn_handlers[] = {
-    {&advertisement_handler, GNUNET_MESSAGE_TYPE_HOSTLIST_ADVERTISEMENT, 0},
-    {NULL, 0, 0}
+  struct GNUNET_MQ_MessageHandler learn_handlers[] = {
+    GNUNET_MQ_hd_var_size (advertisement,
+                           GNUNET_MESSAGE_TYPE_HOSTLIST_ADVERTISEMENT,
+                           struct GNUNET_MessageHeader,
+                           NULL),
+    GNUNET_MQ_handler_end ()
   };
-  static const struct GNUNET_CORE_MessageHandler no_learn_handlers[] = {
-    {NULL, 0, 0}
+  struct GNUNET_MQ_MessageHandler no_learn_handlers[] = {
+    GNUNET_MQ_handler_end ()
   };
-  if ((!bootstrapping) && (!learning)
+  if ((! bootstrapping) && (! learning)
 #if HAVE_MHD
-      && (!provide_hostlist)
+      && (! provide_hostlist)
 #endif
       )
   {
     GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                _
-                ("None of the functions for the hostlist daemon were enabled.  I have no reason to run!\n"));
+                _("None of the functions for the hostlist daemon were enabled.  I have no reason to run!\n"));
     return;
   }
   stats = GNUNET_STATISTICS_create ("hostlist", cfg);
+  if (NULL == stats)
+  {
+    GNUNET_break (0);
+    return;
+  }
   if (bootstrapping)
-    GNUNET_HOSTLIST_client_start (cfg, stats, &client_ch, &client_dh,
-                                  &client_adv_handler, learning);
+    GNUNET_HOSTLIST_client_start (cfg,
+				  stats,
+                                  &client_ch,
+                                  &client_dh,
+                                  &client_adv_handler,
+                                  learning);
   core =
-    GNUNET_CORE_connect (cfg, NULL,
+    GNUNET_CORE_connect (cfg,
+			 NULL,
 			 &core_init,
 			 &connect_handler,
-			 &disconnect_handler, NULL,
-			 GNUNET_NO, NULL,
-			 GNUNET_NO,
+			 &disconnect_handler,
 			 learning ? learn_handlers : no_learn_handlers);
 
 
 #if HAVE_MHD
   if (provide_hostlist)
-    GNUNET_HOSTLIST_server_start (cfg, stats, core, &server_ch, &server_dh,
+    GNUNET_HOSTLIST_server_start (cfg,
+				  stats,
+				  core,
+				  &server_ch,
                                   advertising);
 #endif
-  GNUNET_SCHEDULER_add_delayed (GNUNET_TIME_UNIT_FOREVER_REL,
-				&cleaning_task,
-                                NULL);
+  GNUNET_SCHEDULER_add_shutdown (&cleaning_task,
+				 NULL);
 
   if (NULL == core)
   {
@@ -302,23 +369,26 @@ run (void *cls, char *const *args, const char *cfgfile,
 int
 main (int argc, char *const *argv)
 {
-  static const struct GNUNET_GETOPT_CommandLineOption options[] = {
+  struct GNUNET_GETOPT_CommandLineOption options[] = {
 #if HAVE_MHD
-    {'a', "advertise", NULL,
-     gettext_noop ("advertise our hostlist to other peers"),
-     GNUNET_NO, &GNUNET_GETOPT_set_one, &advertising},
+    GNUNET_GETOPT_option_flag ('a',
+                                  "advertise",
+                                  gettext_noop ("advertise our hostlist to other peers"),
+                                  &advertising),
 #endif
-    {'b', "bootstrap", NULL,
-     gettext_noop
-     ("bootstrap using hostlists (it is highly recommended that you always use this option)"),
-     GNUNET_NO, &GNUNET_GETOPT_set_one, &bootstrapping},
-    {'e', "enable-learning", NULL,
-     gettext_noop ("enable learning about hostlist servers from other peers"),
-     GNUNET_NO, &GNUNET_GETOPT_set_one, &learning},
+    GNUNET_GETOPT_option_flag ('b',
+                                  "bootstrap",
+                                  gettext_noop ("bootstrap using hostlists (it is highly recommended that you always use this option)"),
+                                  &bootstrapping),
+    GNUNET_GETOPT_option_flag ('e',
+                                  "enable-learning",
+                                  gettext_noop ("enable learning about hostlist servers from other peers"),
+                                  &learning),
 #if HAVE_MHD
-    {'p', "provide-hostlist", NULL,
-     gettext_noop ("provide a hostlist server"),
-     GNUNET_NO, &GNUNET_GETOPT_set_one, &provide_hostlist},
+    GNUNET_GETOPT_option_flag ('p',
+                                  "provide-hostlist",
+                                  gettext_noop ("provide a hostlist server"),
+                                  &provide_hostlist),
 #endif
     GNUNET_GETOPT_OPTION_END
   };
@@ -331,8 +401,10 @@ main (int argc, char *const *argv)
   GNUNET_log_setup ("hostlist", "WARNING", NULL);
   ret =
       (GNUNET_OK ==
-       GNUNET_PROGRAM_run (argc, argv, "hostlist",
-                           _("GNUnet hostlist server and client"), options,
+       GNUNET_PROGRAM_run (argc, argv,
+                           "hostlist",
+                           _("GNUnet hostlist server and client"),
+                           options,
                            &run, NULL)) ? 0 : 1;
   GNUNET_free ((void*) argv);
   return ret;
