@@ -1,40 +1,39 @@
 /*
      This file is part of GNUnet.
-     (C) 2001-2012 Christian Grothoff (and other contributing authors)
+     Copyright (C) 2001-2014, 2016 GNUnet e.V.
 
-     GNUnet is free software; you can redistribute it and/or modify
-     it under the terms of the GNU General Public License as published
-     by the Free Software Foundation; either version 3, or (at your
-     option) any later version.
+     GNUnet is free software: you can redistribute it and/or modify it
+     under the terms of the GNU Affero General Public License as published
+     by the Free Software Foundation, either version 3 of the License,
+     or (at your option) any later version.
 
      GNUnet is distributed in the hope that it will be useful, but
      WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-     General Public License for more details.
+     Affero General Public License for more details.
+    
+     You should have received a copy of the GNU Affero General Public License
+     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-     You should have received a copy of the GNU General Public License
-     along with GNUnet; see the file COPYING.  If not, write to the
-     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-     Boston, MA 02111-1307, USA.
+     SPDX-License-Identifier: AGPL3.0-or-later
 */
 
 /**
  * @file peerinfo-tool/gnunet-peerinfo.c
  * @brief Print information about other known peers.
  * @author Christian Grothoff
+ * @author Matthias Wachs
  */
 #include "platform.h"
-#include "gnunet_crypto_lib.h"
-#include "gnunet_configuration_lib.h"
-#include "gnunet_getopt_lib.h"
-#include "gnunet_program_lib.h"
+#include "gnunet_util_lib.h"
 #include "gnunet_hello_lib.h"
 #include "gnunet_transport_service.h"
+#include "gnunet_transport_hello_service.h"
 #include "gnunet_peerinfo_service.h"
 #include "gnunet-peerinfo_plugins.h"
 
 /**
- * How long until we time out during peerinfo iterations?
+ * How long until we time out during address lookup?
  */
 #define TIMEOUT GNUNET_TIME_relative_multiply (GNUNET_TIME_UNIT_SECONDS, 5)
 
@@ -97,22 +96,22 @@ struct PrintContext
   struct AddressRecord *address_list;
 
   /**
-   * Number of completed addresses in 'address_list'.
+   * Number of completed addresses in @e address_list.
    */
   unsigned int num_addresses;
 
   /**
-   * Number of addresses allocated in 'address_list'.
+   * Number of addresses allocated in @e address_list.
    */
   unsigned int address_list_size;
 
   /**
-   * Current offset in 'address_list' (counted down).
+   * Current offset in @e address_list (counted down).
    */
   unsigned int off;
 
   /**
-   * Hello was friend only, GNUNET_YES or GNUNET_NO
+   * Hello was friend only, #GNUNET_YES or #GNUNET_NO
    */
   int friend_only;
 
@@ -177,7 +176,12 @@ static const struct GNUNET_CONFIGURATION_Handle *cfg;
 /**
  * Main state machine task (if active).
  */
-static GNUNET_SCHEDULER_TaskIdentifier tt;
+static struct GNUNET_SCHEDULER_Task *tt;
+
+/**
+ * Pending #GNUNET_TRANSPORT_hello_get() operation.
+ */
+static struct GNUNET_TRANSPORT_HelloGetHandle *gh;
 
 /**
  * Current iterator context (if active, otherwise NULL).
@@ -200,9 +204,14 @@ static struct PrintContext *pc_head;
 static struct PrintContext *pc_tail;
 
 /**
- * Handle to current 'GNUNET_PEERINFO_add_peer' operation.
+ * Handle to current #GNUNET_PEERINFO_add_peer() operation.
  */
-static struct GNUNET_PEERINFO_AddContext *ac;
+static struct GNUNET_MQ_Envelope *ac;
+
+/**
+ * Hello of this peer (if initialized).
+ */
+static struct GNUNET_HELLO_Message *my_hello;
 
 
 /**
@@ -210,17 +219,15 @@ static struct GNUNET_PEERINFO_AddContext *ac;
  * runs the next requested function.
  *
  * @param cls unused
- * @param tc unused
  */
 static void
-state_machine (void *cls,
-	       const struct GNUNET_SCHEDULER_TaskContext *tc);
+state_machine (void *cls);
 
 
 /* ********************* 'get_info' ******************* */
 
 /**
- * Print the collected address information to the console and free 'pc'.
+ * Print the collected address information to the console and free @a pc.
  *
  * @param pc printing context
  */
@@ -236,7 +243,9 @@ dump_pc (struct PrintContext *pc)
   {
     if (NULL != pc->address_list[i].result)
     {
-      printf (_("\tExpires: %s \t %s\n"), GNUNET_STRINGS_absolute_time_to_string(pc->address_list[i].expiration), pc->address_list[i].result);
+      printf (_("\tExpires: %s \t %s\n"),
+              GNUNET_STRINGS_absolute_time_to_string (pc->address_list[i].expiration),
+              pc->address_list[i].result);
       GNUNET_free (pc->address_list[i].result);
     }
   }
@@ -248,7 +257,8 @@ dump_pc (struct PrintContext *pc)
   GNUNET_free (pc);
   if ( (NULL == pc_head) &&
        (NULL == pic) )
-    tt = GNUNET_SCHEDULER_add_now (&state_machine, NULL);
+    tt = GNUNET_SCHEDULER_add_now (&state_machine,
+                                   NULL);
 }
 
 
@@ -260,20 +270,34 @@ dump_pc (struct PrintContext *pc)
  *
  * @param cls closure
  * @param address NULL on error, otherwise 0-terminated printable UTF-8 string
+ * @param res result of the address to string conversion:
+ *        if #GNUNET_OK: address was valid (conversion to
+ *                       string might still have failed)
+ *        if #GNUNET_SYSERR: address is invalid
  */
 static void
-process_resolved_address (void *cls, const char *address)
+process_resolved_address (void *cls,
+                          const char *address,
+                          int res)
 {
-  struct AddressRecord * ar = cls;
+  struct AddressRecord *ar = cls;
   struct PrintContext *pc = ar->pc;
 
   if (NULL != address)
   {
-    if (NULL == ar->result)
+    if (0 != strlen (address))
+    {
+      if (NULL != ar->result)
+        GNUNET_free (ar->result);
       ar->result = GNUNET_strdup (address);
+    }
     return;
   }
   ar->atsc = NULL;
+  if (GNUNET_SYSERR == res)
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                _("Failure: Cannot convert address to string for peer `%s'\n"),
+                GNUNET_i2s (&ar->pc->peer));
   pc->num_addresses++;
   if (pc->num_addresses == pc->address_list_size)
     dump_pc (pc);
@@ -283,13 +307,14 @@ process_resolved_address (void *cls, const char *address)
 /**
  * Iterator callback to go over all addresses and count them.
  *
- * @param cls 'struct PrintContext' with 'off' to increment
+ * @param cls `struct PrintContext *` with `off` to increment
  * @param address the address
  * @param expiration expiration time
- * @return GNUNET_OK to keep the address and continue
+ * @return #GNUNET_OK to keep the address and continue
  */
 static int
-count_address (void *cls, const struct GNUNET_HELLO_Address *address,
+count_address (void *cls,
+               const struct GNUNET_HELLO_Address *address,
                struct GNUNET_TIME_Absolute expiration)
 {
   struct PrintContext *pc = cls;
@@ -305,38 +330,48 @@ count_address (void *cls, const struct GNUNET_HELLO_Address *address,
  * @param cls closure
  * @param address the address
  * @param expiration expiration time
- * @return GNUNET_OK to keep the address and continue
+ * @return #GNUNET_OK to keep the address and continue
  */
 static int
-print_address (void *cls, const struct GNUNET_HELLO_Address *address,
+print_address (void *cls,
+               const struct GNUNET_HELLO_Address *address,
                struct GNUNET_TIME_Absolute expiration)
 {
   struct PrintContext *pc = cls;
   struct AddressRecord *ar;
+
   GNUNET_assert (0 < pc->off);
   ar = &pc->address_list[--pc->off];
   ar->pc = pc;
   ar->expiration = expiration;
-  ar->atsc = GNUNET_TRANSPORT_address_to_string (cfg, address, no_resolve,
-						 GNUNET_TIME_relative_multiply
-						 (GNUNET_TIME_UNIT_SECONDS, 10),
+  GNUNET_asprintf (&ar->result,
+                   "%s:%u:%u",
+                   address->transport_name,
+                   address->address_length,
+                   address->local_info);
+  ar->atsc = GNUNET_TRANSPORT_address_to_string (cfg,
+                                                 address,
+                                                 no_resolve,
+						 TIMEOUT,
 						 &process_resolved_address, ar);
   return GNUNET_OK;
 }
 
 
 /**
- * Print information about the peer.
- * Currently prints the GNUNET_PeerIdentity and the transport address.
+ * Print information about the peer.  Currently prints the `struct
+ * GNUNET_PeerIdentity` and the transport address.
  *
- * @param cls the 'struct PrintContext'
+ * @param cls the `struct PrintContext *`
  * @param peer identity of the peer
  * @param hello addresses of the peer
  * @param err_msg error message
  */
 static void
-print_peer_info (void *cls, const struct GNUNET_PeerIdentity *peer,
-                 const struct GNUNET_HELLO_Message *hello, const char *err_msg)
+print_peer_info (void *cls,
+                 const struct GNUNET_PeerIdentity *peer,
+                 const struct GNUNET_HELLO_Message *hello,
+                 const char *err_msg)
 {
   struct PrintContext *pc;
   int friend_only;
@@ -357,7 +392,8 @@ print_peer_info (void *cls, const struct GNUNET_PeerIdentity *peer,
   friend_only = GNUNET_NO;
   if (NULL != hello)
   	friend_only = GNUNET_HELLO_is_friend_only (hello);
-  if ((GNUNET_YES == be_quiet) || (NULL == hello))
+  if ( (GNUNET_YES == be_quiet) ||
+       (NULL == hello) )
   {
     printf ("%s%s\n",
 	    (GNUNET_YES == friend_only) ? "F2F: " : "",
@@ -381,88 +417,93 @@ print_peer_info (void *cls, const struct GNUNET_PeerIdentity *peer,
   }
   pc->address_list_size = pc->off;
   pc->address_list = GNUNET_malloc (sizeof (struct AddressRecord) * pc->off);
-  GNUNET_HELLO_iterate_addresses (hello, GNUNET_NO,
-				  &print_address, pc);
+  GNUNET_HELLO_iterate_addresses (hello,
+                                  GNUNET_NO,
+				  &print_address,
+                                  pc);
 }
 
 /* ************************* DUMP Hello  ************************** */
 
-static int count_addr(void *cls,
-										 const struct GNUNET_HELLO_Address *address,
-										 struct GNUNET_TIME_Absolute expiration)
+/**
+ * Count the number of addresses in the HELLO.
+ *
+ * @param cls pointer to an `int *` used for the counter
+ * @param address an address to count
+ * @param expiration (unused)
+ * @return #GNUNET_OK
+ */
+static int
+count_addr (void *cls,
+            const struct GNUNET_HELLO_Address *address,
+            struct GNUNET_TIME_Absolute expiration)
 {
-	int *c = cls;
-  (*c) ++;
+  int *c = cls;
+
+  (*c)++;
   return GNUNET_OK;
 }
 
+
 /**
- * Write Hello of my peer to a file.
+ * Write HELLO of my peer to a file.
  *
- * @param cls the 'struct GetUriContext'
+ * @param cls the `struct GetUriContext *`
  * @param peer identity of the peer (unused)
  * @param hello addresses of the peer
  * @param err_msg error message
  */
 static void
-dump_my_hello (void *cls, const struct GNUNET_PeerIdentity *peer,
-              const struct GNUNET_HELLO_Message *hello,
-	      const char *err_msg)
+dump_my_hello ()
 {
-	unsigned int size;
-	unsigned int c_addr;
-  if (peer == NULL)
-  {
-    pic = NULL;
-    if (err_msg != NULL)
-      FPRINTF (stderr,
-	       _("Error in communication with PEERINFO service: %s\n"),
-	       err_msg);
-    tt = GNUNET_SCHEDULER_add_now (&state_machine, NULL);
-    return;
-  }
+  unsigned int size;
+  unsigned int c_addr;
 
-  if (NULL == hello)
-  {
-		FPRINTF (stderr,
-			 _("Failure: Did not receive %s\n"), "HELLO");
-    return;
-  }
-
-  size = GNUNET_HELLO_size (hello);
+  size = GNUNET_HELLO_size (my_hello);
   if (0 == size)
   {
-  		FPRINTF (stderr,
-  			 _("Failure: Received invalid %s\n"), "HELLO");
-      return;
+    FPRINTF (stderr,
+             _("Failure: Received invalid %s\n"),
+             "HELLO");
+    return;
   }
-  if (GNUNET_SYSERR == GNUNET_DISK_fn_write (dump_hello, hello, size,
+  if (GNUNET_SYSERR ==
+      GNUNET_DISK_fn_write (dump_hello,
+                            my_hello,
+                            size,
                             GNUNET_DISK_PERM_USER_READ |
                             GNUNET_DISK_PERM_USER_WRITE |
                             GNUNET_DISK_PERM_GROUP_READ |
                             GNUNET_DISK_PERM_OTHER_READ))
   {
-  		FPRINTF (stderr, _("Failed to write HELLO with %u bytes to file `%s'\n"),
-  			 size, dump_hello);
-  		if (0 != UNLINK (dump_hello))
-  		GNUNET_log_strerror_file (GNUNET_ERROR_TYPE_WARNING |
-                              GNUNET_ERROR_TYPE_BULK, "unlink", dump_hello);
+    FPRINTF (stderr,
+             _("Failed to write HELLO with %u bytes to file `%s'\n"),
+             size,
+             dump_hello);
+    if (0 != UNLINK (dump_hello))
+      GNUNET_log_strerror_file (GNUNET_ERROR_TYPE_WARNING |
+                                GNUNET_ERROR_TYPE_BULK,
+                                "unlink",
+                                dump_hello);
 
   }
   c_addr = 0;
-  GNUNET_HELLO_iterate_addresses (hello, GNUNET_NO, count_addr, &c_addr);
+  GNUNET_HELLO_iterate_addresses (my_hello,
+                                  GNUNET_NO,
+                                  count_addr,
+                                  &c_addr);
 
-  if (!be_quiet)
+  if (! be_quiet)
   {
-  		FPRINTF (stderr,
-  			 _("Wrote %s HELLO containing %u addresses with %u bytes to file `%s'\n"),
-  			 (GNUNET_YES == GNUNET_HELLO_is_friend_only(hello)) ? "friend-only": "public",
-  					c_addr, size, dump_hello);
+    FPRINTF (stderr,
+             _("Wrote %s HELLO containing %u addresses with %u bytes to file `%s'\n"),
+             (GNUNET_YES == GNUNET_HELLO_is_friend_only (my_hello)) ? "friend-only": "public",
+             c_addr,
+             size,
+             dump_hello);
   }
-
   GNUNET_free (dump_hello);
   dump_hello = NULL;
-
 }
 
 
@@ -472,20 +513,23 @@ dump_my_hello (void *cls, const struct GNUNET_PeerIdentity *peer,
 /**
  * Print URI of the peer.
  *
- * @param cls the 'struct GetUriContext'
+ * @param cls the `struct GetUriContext *`
  * @param peer identity of the peer (unused)
  * @param hello addresses of the peer
  * @param err_msg error message
  */
 static void
-print_my_uri (void *cls, const struct GNUNET_PeerIdentity *peer,
+print_my_uri (void *cls,
+              const struct GNUNET_PeerIdentity *peer,
               const struct GNUNET_HELLO_Message *hello,
 	      const char *err_msg)
 {
-  if (peer == NULL)
+  char *uri;
+
+  if (NULL == peer)
   {
     pic = NULL;
-    if (err_msg != NULL)
+    if (NULL != err_msg)
       FPRINTF (stderr,
 	       _("Error in communication with PEERINFO service: %s\n"),
 	       err_msg);
@@ -495,9 +539,12 @@ print_my_uri (void *cls, const struct GNUNET_PeerIdentity *peer,
 
   if (NULL == hello)
     return;
-  char *uri = GNUNET_HELLO_compose_uri(hello, &GPI_plugins_find);
-  if (NULL != uri) {
-    printf ("%s\n", (const char *) uri);
+  uri = GNUNET_HELLO_compose_uri (hello,
+                                  &GPI_plugins_find);
+  if (NULL != uri)
+  {
+    printf ("%s\n",
+            (const char *) uri);
     GNUNET_free (uri);
   }
 }
@@ -507,20 +554,14 @@ print_my_uri (void *cls, const struct GNUNET_PeerIdentity *peer,
 
 
 /**
- * Continuation called from 'GNUNET_PEERINFO_add_peer'
+ * Continuation called from #GNUNET_PEERINFO_add_peer()
  *
  * @param cls closure, NULL
- * @param emsg error message, NULL on success
  */
 static void
-add_continuation (void *cls,
-		  const char *emsg)
+add_continuation (void *cls)
 {
   ac = NULL;
-  if (NULL != emsg)
-    fprintf (stderr,
-	     _("Failure adding HELLO: %s\n"),
-	     emsg);
   tt = GNUNET_SCHEDULER_add_now (&state_machine, NULL);
 }
 
@@ -530,28 +571,32 @@ add_continuation (void *cls,
  * database.
  *
  * @param put_uri URI string to parse
- * @return GNUNET_OK on success, GNUNET_SYSERR if the URI was invalid, GNUNET_NO on other errors
+ * @return #GNUNET_OK on success,
+ *         #GNUNET_SYSERR if the URI was invalid,
+ *         #GNUNET_NO on other errors
  */
 static int
 parse_hello_uri (const char *put_uri)
 {
   struct GNUNET_HELLO_Message *hello = NULL;
 
-  int ret = GNUNET_HELLO_parse_uri(put_uri, &my_peer_identity.public_key,
-				   &hello, &GPI_plugins_find);
+  int ret = GNUNET_HELLO_parse_uri (put_uri,
+                                    &my_peer_identity.public_key,
+                                    &hello,
+                                    &GPI_plugins_find);
 
-  if (NULL != hello) {
+  if (NULL != hello)
+  {
     /* WARNING: this adds the address from URI WITHOUT verification! */
     if (GNUNET_OK == ret)
-      ac = GNUNET_PEERINFO_add_peer (peerinfo, hello, &add_continuation, NULL);
+      ac = GNUNET_PEERINFO_add_peer (peerinfo,
+                                     hello,
+                                     &add_continuation,
+                                     NULL);
     else
       tt = GNUNET_SCHEDULER_add_now (&state_machine, NULL);
     GNUNET_free (hello);
   }
-
-  /* wait 1s to give peerinfo operation a chance to succeed */
-  /* FIXME: current peerinfo API sucks to require this; not to mention
-     that we get no feedback to determine if the operation actually succeeded */
   return ret;
 }
 
@@ -564,11 +609,9 @@ parse_hello_uri (const char *put_uri)
  * runs the next requested function.
  *
  * @param cls unused
- * @param tc scheduler context
  */
 static void
-shutdown_task (void *cls,
-	       const struct GNUNET_SCHEDULER_TaskContext *tc)
+shutdown_task (void *cls)
 {
   struct PrintContext *pc;
   struct AddressRecord *ar;
@@ -576,19 +619,24 @@ shutdown_task (void *cls,
 
   if (NULL != ac)
   {
-    GNUNET_PEERINFO_add_peer_cancel (ac);
+    GNUNET_MQ_send_cancel (ac);
     ac = NULL;
   }
-  if (GNUNET_SCHEDULER_NO_TASK != tt)
+  if (NULL != tt)
   {
     GNUNET_SCHEDULER_cancel (tt);
-    tt = GNUNET_SCHEDULER_NO_TASK;
+    tt = NULL;
   }
   if (NULL != pic)
   {
     GNUNET_PEERINFO_iterate_cancel (pic);
     pic = NULL;
   }
+  if (NULL != gh)
+  {
+    GNUNET_TRANSPORT_hello_get_cancel (gh);
+    gh = NULL;
+  }
   while (NULL != (pc = pc_head))
   {
     GNUNET_CONTAINER_DLL_remove (pc_head,
@@ -613,58 +661,41 @@ shutdown_task (void *cls,
     GNUNET_PEERINFO_disconnect (peerinfo);
     peerinfo = NULL;
   }
+  if (NULL != my_hello)
+  {
+    GNUNET_free (my_hello);
+    my_hello = NULL;
+  }
 }
 
+
 /**
- * Function called with the result of the check if the 'peerinfo'
- * service is running.
+ * Function called with our peer's HELLO message.
+ * Used to obtain our peer's public key.
  *
- * @param cls closure with our configuration
- * @param result #GNUNET_YES if transport is running
+ * @param cls NULL
+ * @param hello the HELLO message
  */
 static void
-testservice_task (void *cls, int result)
+hello_callback (void *cls,
+                const struct GNUNET_MessageHeader *hello)
 {
-  struct GNUNET_CRYPTO_EddsaPrivateKey *priv;
-  char *fn;
-
-  if (GNUNET_YES != result)
+  if (NULL == hello)
   {
-    FPRINTF (stderr, _("Service `%s' is not running, please start GNUnet\n"), "peerinfo");
+    fprintf (stderr,
+             "Failed to get my own HELLO from this peer!\n");
+    GNUNET_SCHEDULER_shutdown ();
     return;
   }
-
-  if (NULL == (peerinfo = GNUNET_PEERINFO_connect (cfg)))
-  {
-    FPRINTF (stderr, "%s",  _("Could not access PEERINFO service.  Exiting.\n"));
-    return;
-  }
-  if ( (GNUNET_YES == get_self) || (GNUNET_YES == get_uri) || (NULL != dump_hello) )
-  {
-    /* load private key */
-    if (GNUNET_OK != GNUNET_CONFIGURATION_get_value_filename (cfg, "PEER",
-                                                              "PRIVATE_KEY",
-                                                               &fn))
-    {
-      GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR, "PEER", "PRIVATE_KEY");
-      return;
-    }
-    if (NULL == (priv = GNUNET_CRYPTO_eddsa_key_create_from_file (fn)))
-    {
-      FPRINTF (stderr, _("Loading hostkey from `%s' failed.\n"), fn);
-      GNUNET_free (fn);
-      return;
-    }
-    GNUNET_free (fn);
-    GNUNET_CRYPTO_eddsa_key_get_public (priv,
-                                        &my_peer_identity.public_key);
-    GNUNET_free (priv);
-  }
-
+  my_hello = (struct GNUNET_HELLO_Message *) GNUNET_copy_message (hello);
+  GNUNET_assert (GNUNET_OK ==
+                 GNUNET_HELLO_get_id (my_hello,
+                                      &my_peer_identity));
+  GNUNET_TRANSPORT_hello_get_cancel (gh);
+  gh = NULL;
+  if (NULL != dump_hello)
+    dump_my_hello ();
   tt = GNUNET_SCHEDULER_add_now (&state_machine, NULL);
-  GNUNET_SCHEDULER_add_delayed (GNUNET_TIME_UNIT_FOREVER_REL,
-                                &shutdown_task,
-                                NULL);
 }
 
 
@@ -677,13 +708,16 @@ testservice_task (void *cls, int result)
  * @param c configuration
  */
 static void
-run (void *cls, char *const *args, const char *cfgfile,
+run (void *cls,
+     char *const *args,
+     const char *cfgfile,
      const struct GNUNET_CONFIGURATION_Handle *c)
 {
   cfg = c;
   if ( (NULL != args[0]) &&
        (NULL == put_uri) &&
-       (args[0] == strcasestr (args[0], "gnunet://hello/")) )
+       (args[0] == strcasestr (args[0],
+			       "gnunet://hello/")) )
   {
     put_uri = GNUNET_strdup (args[0]);
     args++;
@@ -695,9 +729,29 @@ run (void *cls, char *const *args, const char *cfgfile,
 	     args[0]);
     return;
   }
-
-  GNUNET_CLIENT_service_test ("peerinfo", cfg, GNUNET_TIME_UNIT_SECONDS,
-      &testservice_task, (void *) cfg);
+  if (NULL == (peerinfo = GNUNET_PEERINFO_connect (cfg)))
+  {
+    FPRINTF (stderr,
+             "%s",
+             "Could not access PEERINFO service.  Exiting.\n");
+    return;
+  }
+  if ( (GNUNET_YES == get_self) ||
+       (GNUNET_YES == get_uri) ||
+       (NULL != dump_hello) )
+  {
+    gh = GNUNET_TRANSPORT_hello_get (cfg,
+				     GNUNET_TRANSPORT_AC_ANY,
+                                     &hello_callback,
+                                     NULL);
+  }
+  else
+  {
+    tt = GNUNET_SCHEDULER_add_now (&state_machine,
+				   NULL);
+  }
+  GNUNET_SCHEDULER_add_shutdown (&shutdown_task,
+				 NULL);
 }
 
 
@@ -706,13 +760,11 @@ run (void *cls, char *const *args, const char *cfgfile,
  * runs the next requested function.
  *
  * @param cls unused
- * @param tc scheduler context
  */
 static void
-state_machine (void *cls,
-	       const struct GNUNET_SCHEDULER_TaskContext *tc)
+state_machine (void *cls)
 {
-  tt = GNUNET_SCHEDULER_NO_TASK;
+  tt = NULL;
 
   if (NULL != put_uri)
   {
@@ -731,9 +783,11 @@ state_machine (void *cls,
   {
     get_info = GNUNET_NO;
     GPI_plugins_load (cfg);
-    pic = GNUNET_PEERINFO_iterate (peerinfo, include_friend_only, NULL,
-				   TIMEOUT,
-				   &print_peer_info, NULL);
+    pic = GNUNET_PEERINFO_iterate (peerinfo,
+                                   include_friend_only,
+                                   NULL,
+				   &print_peer_info,
+				   NULL);
   }
   else if (GNUNET_YES == get_self)
   {
@@ -744,32 +798,32 @@ state_machine (void *cls,
     else
       printf (_("I am peer `%s'.\n"),
 	      GNUNET_i2s_full (&my_peer_identity));
-    tt = GNUNET_SCHEDULER_add_now (&state_machine, NULL);
+    tt = GNUNET_SCHEDULER_add_now (&state_machine,
+				   NULL);
   }
   else if (GNUNET_YES == get_uri)
   {
     GPI_plugins_load (cfg);
-    pic = GNUNET_PEERINFO_iterate (peerinfo, include_friend_only, &my_peer_identity,
-				   TIMEOUT, &print_my_uri, NULL);
+    pic = GNUNET_PEERINFO_iterate (peerinfo,
+                                   include_friend_only,
+                                   &my_peer_identity,
+                                   &print_my_uri,
+				   NULL);
     get_uri = GNUNET_NO;
   }
-  else if (NULL != dump_hello)
-  {
-    pic = GNUNET_PEERINFO_iterate (peerinfo, include_friend_only, &my_peer_identity,
-				   TIMEOUT, &dump_my_hello, NULL);
-  }
   else if (GNUNET_YES == default_operation)
   {
-  	/* default operation list all */
-  	default_operation = GNUNET_NO;
-  	get_info = GNUNET_YES;
-  	tt = GNUNET_SCHEDULER_add_now (&state_machine, NULL);
+    /* default operation list all */
+    default_operation = GNUNET_NO;
+    get_info = GNUNET_YES;
+    tt = GNUNET_SCHEDULER_add_now (&state_machine,
+				   NULL);
   }
   else
   {
-  	GNUNET_SCHEDULER_shutdown ();
+    GNUNET_SCHEDULER_shutdown ();
   }
-	default_operation = GNUNET_NO;
+  default_operation = GNUNET_NO;
 }
 
 
@@ -783,43 +837,67 @@ state_machine (void *cls,
 int
 main (int argc, char *const *argv)
 {
-	default_operation = GNUNET_YES;
-  static const struct GNUNET_GETOPT_CommandLineOption options[] = {
-    {'n', "numeric", NULL,
-     gettext_noop ("don't resolve host names"),
-     0, &GNUNET_GETOPT_set_one, &no_resolve},
-    {'q', "quiet", NULL,
-     gettext_noop ("output only the identity strings"),
-     0, &GNUNET_GETOPT_set_one, &be_quiet},
-    {'f', "friends", NULL,
-     gettext_noop ("include friend-only information"),
-     0, &GNUNET_GETOPT_set_one, &include_friend_only},
-    {'s', "self", NULL,
-     gettext_noop ("output our own identity only"),
-     0, &GNUNET_GETOPT_set_one, &get_self},
-    {'i', "info", NULL,
-     gettext_noop ("list all known peers"),
-     0, &GNUNET_GETOPT_set_one, &get_info},
-    {'d', "dump-hello", NULL,
-     gettext_noop ("dump hello to file"),
-     1, &GNUNET_GETOPT_set_string, &dump_hello},
-    {'g', "get-hello", NULL,
-     gettext_noop ("also output HELLO uri(s)"),
-     0, &GNUNET_GETOPT_set_one, &get_uri},
-    {'p', "put-hello", "HELLO",
-     gettext_noop ("add given HELLO uri to the database"),
-     1, &GNUNET_GETOPT_set_string, &put_uri},
+  struct GNUNET_GETOPT_CommandLineOption options[] = {
+    GNUNET_GETOPT_option_flag ('n',
+                                  "numeric",
+                                  gettext_noop ("don't resolve host names"),
+                                  &no_resolve),
+
+    GNUNET_GETOPT_option_flag ('q',
+                                  "quiet",
+                                  gettext_noop ("output only the identity strings"),
+                                  &be_quiet),
+    GNUNET_GETOPT_option_flag ('f',
+                                  "friends",
+                                  gettext_noop ("include friend-only information"),
+                                  &include_friend_only),
+
+    GNUNET_GETOPT_option_flag ('s',
+                                  "self",
+                                  gettext_noop ("output our own identity only"),
+                                  &get_self),
+    
+    GNUNET_GETOPT_option_flag ('i',
+                                  "info",
+                                  gettext_noop ("list all known peers"),
+                                  &get_info),
+
+    GNUNET_GETOPT_option_string ('d',
+                                 "dump-hello",
+                                 NULL,
+                                 gettext_noop ("dump hello to file"),
+                                 &dump_hello),
+
+    GNUNET_GETOPT_option_flag ('g',
+                                  "get-hello",
+                                  gettext_noop ("also output HELLO uri(s)"),
+                                  &get_uri),
+
+    GNUNET_GETOPT_option_string ('p',
+                                 "put-hello",
+                                 "HELLO",
+                                 gettext_noop ("add given HELLO uri to the database"),
+                                 &put_uri),
+
     GNUNET_GETOPT_OPTION_END
   };
   int ret;
 
-  if (GNUNET_OK != GNUNET_STRINGS_get_utf8_args (argc, argv, &argc, &argv))
+  default_operation = GNUNET_YES;
+  if (GNUNET_OK != GNUNET_STRINGS_get_utf8_args (argc,
+                                                 argv,
+                                                 &argc,
+						 &argv))
     return 2;
 
   ret = (GNUNET_OK ==
-	 GNUNET_PROGRAM_run (argc, argv, "gnunet-peerinfo",
+	 GNUNET_PROGRAM_run (argc,
+			     argv,
+			     "gnunet-peerinfo",
 			     gettext_noop ("Print information about peers."),
-			     options, &run, NULL)) ? 0 : 1;
+			     options,
+			     &run,
+			     NULL)) ? 0 : 1;
   GNUNET_free ((void*) argv);
   return ret;
 }
