@@ -1,21 +1,21 @@
 /*
   This file is part of GNUnet.
-  (C) 2013 Christian Grothoff (and other contributing authors)
+  Copyright (C) 2013, 2014, 2016 GNUnet e.V.
 
-  GNUnet is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public Licerevocation as published
-  by the Free Software Foundation; either version 3, or (at your
-  option) any later version.
+  GNUnet is free software: you can redistribute it and/or modify it
+  under the terms of the GNU Affero General Public License as published
+  by the Free Software Foundation, either version 3 of the License,
+  or (at your option) any later version.
 
   GNUnet is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public Licerevocation for more details.
+  Affero General Public License for more details.
+ 
+  You should have received a copy of the GNU Affero General Public License
+  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-  You should have received a copy of the GNU General Public Licerevocation
-  along with GNUnet; see the file COPYING.  If not, write to the
-  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-  Boston, MA 02111-1307, USA.
+     SPDX-License-Identifier: AGPL3.0-or-later
  */
 
 /**
@@ -38,6 +38,7 @@
 #include "platform.h"
 #include <math.h>
 #include "gnunet_util_lib.h"
+#include "gnunet_block_lib.h"
 #include "gnunet_constants.h"
 #include "gnunet_protocols.h"
 #include "gnunet_signatures.h"
@@ -68,7 +69,7 @@ struct PeerEntry
   /**
    * Tasked used to trigger the set union operation.
    */
-  GNUNET_SCHEDULER_TaskIdentifier transmit_task;
+  struct GNUNET_SCHEDULER_Task *transmit_task;
 
   /**
    * Handle to active set union operation (over revocation sets).
@@ -114,16 +115,6 @@ static struct GNUNET_CONTAINER_MultiPeerMap *peers;
  */
 static struct GNUNET_PeerIdentity my_identity;
 
-/**
- * Handle to this serivce's server.
- */
-static struct GNUNET_SERVER_Handle *srv;
-
-/**
- * Notification context for convenient sending of replies to the clients.
- */
-static struct GNUNET_SERVER_NotificationContext *nc;
-
 /**
  * File handle for the revocation database.
  */
@@ -146,6 +137,27 @@ static unsigned long long revocation_work_required;
 static struct GNUNET_HashCode revocation_set_union_app_id;
 
 
+/**
+ * Create a new PeerEntry and add it to the peers multipeermap.
+ *
+ * @param peer the peer identity
+ * @return a pointer to the new PeerEntry
+ */
+static struct PeerEntry *
+new_peer_entry(const struct GNUNET_PeerIdentity *peer)
+{
+  struct PeerEntry *peer_entry;
+
+  peer_entry = GNUNET_new (struct PeerEntry);
+  peer_entry->id = *peer;
+  GNUNET_assert (GNUNET_OK ==
+                 GNUNET_CONTAINER_multipeermap_put (peers,
+                                                    &peer_entry->id,
+                                                    peer_entry,
+                                                    GNUNET_CONTAINER_MULTIHASHMAPOPTION_UNIQUE_ONLY));
+  return peer_entry;
+}
+
 
 /**
  * An revoke message has been received, check that it is well-formed.
@@ -180,42 +192,71 @@ verify_revoke_message (const struct RevokeMessage *rm)
 }
 
 
+/**
+ * Handle client connecting to the service.
+ *
+ * @param cls NULL
+ * @param client the new client
+ * @param mq the message queue of @a client
+ * @return @a client
+ */
+static void *
+client_connect_cb (void *cls,
+		   struct GNUNET_SERVICE_Client *client,
+		   struct GNUNET_MQ_Handle *mq)
+{
+  return client;
+}
+
+
+/**
+ * Handle client connecting to the service.
+ *
+ * @param cls NULL
+ * @param client the new client
+ * @param app_cls must alias @a client
+ */
+static void
+client_disconnect_cb (void *cls,
+		      struct GNUNET_SERVICE_Client *client,
+		      void *app_cls)
+{
+  GNUNET_assert (client == app_cls);
+}
+
+
 /**
  * Handle QUERY message from client.
  *
- * @param cls unused
- * @param client who sent the message
- * @param message the message received
+ * @param cls client who sent the message
+ * @param qm the message received
  */
 static void
 handle_query_message (void *cls,
-		      struct GNUNET_SERVER_Client *client,
-                      const struct GNUNET_MessageHeader *message)
+                      const struct QueryMessage *qm)
 {
-  const struct QueryMessage *qm = (const struct QueryMessage *) message;
-  struct QueryResponseMessage qrm;
+  struct GNUNET_SERVICE_Client *client = cls;
+  struct GNUNET_MQ_Envelope *env;
+  struct QueryResponseMessage *qrm;
   struct GNUNET_HashCode hc;
   int res;
 
   GNUNET_CRYPTO_hash (&qm->key,
                       sizeof (struct GNUNET_CRYPTO_EcdsaPublicKey),
                       &hc);
-  res = GNUNET_CONTAINER_multihashmap_contains (revocation_map, &hc);
+  res = GNUNET_CONTAINER_multihashmap_contains (revocation_map,
+                                                &hc);
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               (GNUNET_NO == res)
 	      ? "Received revocation check for valid key `%s' from client\n"
               : "Received revocation check for revoked key `%s' from client\n",
               GNUNET_h2s (&hc));
-  qrm.header.size = htons (sizeof (struct QueryResponseMessage));
-  qrm.header.type = htons (GNUNET_MESSAGE_TYPE_REVOCATION_QUERY_RESPONSE);
-  qrm.is_valid = htonl ((GNUNET_YES == res) ? GNUNET_NO : GNUNET_YES);
-  GNUNET_SERVER_notification_context_add (nc,
-                                          client);
-  GNUNET_SERVER_notification_context_unicast (nc,
-                                              client,
-                                              &qrm.header,
-                                              GNUNET_NO);
-  GNUNET_SERVER_receive_done (client, GNUNET_OK);
+  env = GNUNET_MQ_msg (qrm,
+		       GNUNET_MESSAGE_TYPE_REVOCATION_QUERY_RESPONSE);
+  qrm->is_valid = htonl ((GNUNET_YES == res) ? GNUNET_NO : GNUNET_YES);
+  GNUNET_MQ_send (GNUNET_SERVICE_client_get_mq (client),
+		  env);
+  GNUNET_SERVICE_client_continue (client);
 }
 
 
@@ -237,10 +278,18 @@ do_flood (void *cls,
   struct GNUNET_MQ_Envelope *e;
   struct RevokeMessage *cp;
 
-  e = GNUNET_MQ_msg (cp, GNUNET_MESSAGE_TYPE_REVOCATION_REVOKE);
+  if (NULL == pe->mq)
+    return GNUNET_OK; /* peer connected to us via SET,
+			 but we have no direct CORE
+			 connection for flooding */
+  e = GNUNET_MQ_msg (cp,
+                     GNUNET_MESSAGE_TYPE_REVOCATION_REVOKE);
   *cp = *rm;
-  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "Flooding revocation to `%s'\n", GNUNET_i2s (target));
-  GNUNET_MQ_send (pe->mq, e);
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+              "Flooding revocation to `%s'\n",
+              GNUNET_i2s (target));
+  GNUNET_MQ_send (pe->mq,
+                  e);
   return GNUNET_OK;
 }
 
@@ -268,8 +317,8 @@ publicize_rm (const struct RevokeMessage *rm)
       GNUNET_CONTAINER_multihashmap_contains (revocation_map,
                                               &hc))
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-                _("Duplicate revocation received from peer. Ignored.\n"));
+    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+                "Duplicate revocation received from peer. Ignored.\n");
     return GNUNET_OK;
   }
   if (GNUNET_OK !=
@@ -304,12 +353,13 @@ publicize_rm (const struct RevokeMessage *rm)
                                                    GNUNET_CONTAINER_MULTIHASHMAPOPTION_UNIQUE_ONLY));
   /* add to set for future connections */
   e.size = htons (rm->header.size);
-  e.type = 0;
+  e.element_type = GNUNET_BLOCK_TYPE_REVOCATION;
   e.data = rm;
   if (GNUNET_OK !=
       GNUNET_SET_add_element (revocation_set,
                               &e,
-                              NULL, NULL))
+                              NULL,
+                              NULL))
   {
     GNUNET_break (0);
     return GNUNET_OK;
@@ -330,38 +380,32 @@ publicize_rm (const struct RevokeMessage *rm)
 /**
  * Handle REVOKE message from client.
  *
- * @param cls unused
- * @param client who sent the message
- * @param message the message received
+ * @param cls client who sent the message
+ * @param rm the message received
  */
 static void
 handle_revoke_message (void *cls,
-                       struct GNUNET_SERVER_Client *client,
-                       const struct GNUNET_MessageHeader *message)
+                       const struct RevokeMessage *rm)
 {
-  const struct RevokeMessage *rm;
-  struct RevocationResponseMessage rrm;
+  struct GNUNET_SERVICE_Client *client = cls;
+  struct GNUNET_MQ_Envelope *env;
+  struct RevocationResponseMessage *rrm;
   int ret;
 
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
 	      "Received REVOKE message from client\n");
-  rm = (const struct RevokeMessage *) message;
   if (GNUNET_SYSERR == (ret = publicize_rm (rm)))
   {
     GNUNET_break_op (0);
-    GNUNET_SERVER_receive_done (client, GNUNET_SYSERR);
+    GNUNET_SERVICE_client_drop (client);
     return;
   }
-  rrm.header.size = htons (sizeof (struct RevocationResponseMessage));
-  rrm.header.type = htons (GNUNET_MESSAGE_TYPE_REVOCATION_REVOKE_RESPONSE);
-  rrm.is_valid = htonl ((GNUNET_OK == ret) ? GNUNET_NO : GNUNET_YES);
-  GNUNET_SERVER_notification_context_add (nc,
-                                          client);
-  GNUNET_SERVER_notification_context_unicast (nc,
-                                              client,
-                                              &rrm.header,
-                                              GNUNET_NO);
-  GNUNET_SERVER_receive_done (client, GNUNET_OK);
+  env = GNUNET_MQ_msg (rrm,
+		       GNUNET_MESSAGE_TYPE_REVOCATION_REVOKE_RESPONSE);
+  rrm->is_valid = htonl ((GNUNET_OK == ret) ? GNUNET_NO : GNUNET_YES);
+  GNUNET_MQ_send (GNUNET_SERVICE_client_get_mq (client),
+		  env);
+  GNUNET_SERVICE_client_continue (client);
 }
 
 
@@ -369,25 +413,19 @@ handle_revoke_message (void *cls,
  * Core handler for flooded revocation messages.
  *
  * @param cls closure unused
- * @param message message
- * @param peer peer identity this message is from (ignored)
+ * @param rm revocation message
  */
-static int
-handle_p2p_revoke_message (void *cls,
-			   const struct GNUNET_PeerIdentity *peer,
-			   const struct GNUNET_MessageHeader *message)
+static void
+handle_p2p_revoke (void *cls,
+		   const struct RevokeMessage *rm)
 {
-  const struct RevokeMessage *rm;
-
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-	      "Received REVOKE message from peer\n");
-  rm = (const struct RevokeMessage *) message;
-  GNUNET_break_op (GNUNET_SYSERR != publicize_rm (rm));
-  return GNUNET_OK;
+	      "Received REVOKE message\n");
+  GNUNET_break_op (GNUNET_SYSERR !=
+		   publicize_rm (rm));
 }
 
 
-
 /**
  * Callback for set operation results. Called for each element in the
  * result set.  Each element contains a revocation, which we should
@@ -395,11 +433,13 @@ handle_p2p_revoke_message (void *cls,
  *
  * @param cls closure
  * @param element a result element, only valid if status is #GNUNET_SET_STATUS_OK
+ * @param current_size current set size
  * @param status see `enum GNUNET_SET_Status`
  */
 static void
 add_revocation (void *cls,
                 const struct GNUNET_SET_Element *element,
+                uint64_t current_size,
                 enum GNUNET_SET_Status status)
 {
   struct PeerEntry *peer_entry = cls;
@@ -413,36 +453,39 @@ add_revocation (void *cls,
       GNUNET_break_op (0);
       return;
     }
-    if (0 != element->type)
+    if (GNUNET_BLOCK_TYPE_REVOCATION != element->element_type)
     {
       GNUNET_STATISTICS_update (stats,
-                                "# unsupported revocations received via set union",
-                                1, GNUNET_NO);
+                                gettext_noop ("# unsupported revocations received via set union"),
+                                1,
+                                GNUNET_NO);
       return;
     }
     rm = element->data;
-    (void) handle_p2p_revoke_message (NULL,
-                                      &peer_entry->id,
-                                      &rm->header);
+    (void) handle_p2p_revoke (NULL,
+			      rm);
     GNUNET_STATISTICS_update (stats,
-                              "# revocation messages received via set union",
+                              gettext_noop ("# revocation messages received via set union"),
                               1, GNUNET_NO);
     break;
-  case GNUNET_SET_STATUS_TIMEOUT:
   case GNUNET_SET_STATUS_FAILURE:
     GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
                 _("Error computing revocation set union with %s\n"),
                 GNUNET_i2s (&peer_entry->id));
     peer_entry->so = NULL;
-    GNUNET_STATISTICS_update (stats, "# revocation set unions failed",
-                              1, GNUNET_NO);
+    GNUNET_STATISTICS_update (stats,
+                              gettext_noop ("# revocation set unions failed"),
+                              1,
+                              GNUNET_NO);
     break;
   case GNUNET_SET_STATUS_HALF_DONE:
     break;
   case GNUNET_SET_STATUS_DONE:
     peer_entry->so = NULL;
-    GNUNET_STATISTICS_update (stats, "# revocation set unions completed",
-                              1, GNUNET_NO);
+    GNUNET_STATISTICS_update (stats,
+                              gettext_noop ("# revocation set unions completed"),
+                              1,
+                              GNUNET_NO);
     break;
   default:
     GNUNET_break (0);
@@ -456,26 +499,22 @@ add_revocation (void *cls,
  * run the set operation on the revocation certificates.
  *
  * @param cls NULL
- * @param tc scheduler context (unused)
  */
 static void
-transmit_task_cb (void *cls,
-                  const struct GNUNET_SCHEDULER_TaskContext *tc)
+transmit_task_cb (void *cls)
 {
   struct PeerEntry *peer_entry = cls;
-  uint16_t salt;
 
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Starting set exchange with peer `%s'\n",
               GNUNET_i2s (&peer_entry->id));
-
-  salt = (uint16_t) GNUNET_CRYPTO_random_u32 (GNUNET_CRYPTO_QUALITY_WEAK,
-                                              UINT16_MAX);
-  peer_entry->transmit_task = GNUNET_SCHEDULER_NO_TASK;
+  peer_entry->transmit_task = NULL;
+  GNUNET_assert (NULL == peer_entry->so);
   peer_entry->so = GNUNET_SET_prepare (&peer_entry->id,
                                        &revocation_set_union_app_id,
-                                       NULL, salt,
+                                       NULL,
                                        GNUNET_SET_RESULT_ADDED,
+                                       (struct GNUNET_SET_Option[]) {{ 0 }},
                                        &add_revocation,
                                        peer_entry);
   if (GNUNET_OK !=
@@ -497,38 +536,48 @@ transmit_task_cb (void *cls,
  * @param cls closure
  * @param peer peer identity this notification is about
  */
-static void
+static void *
 handle_core_connect (void *cls,
-		     const struct GNUNET_PeerIdentity *peer)
+		     const struct GNUNET_PeerIdentity *peer,
+		     struct GNUNET_MQ_Handle *mq)
 {
   struct PeerEntry *peer_entry;
   struct GNUNET_HashCode my_hash;
   struct GNUNET_HashCode peer_hash;
 
-  if (0 == memcmp(peer, &my_identity, sizeof (my_identity)))
-      return;
+  if (0 == memcmp (peer,
+                   &my_identity,
+                   sizeof (my_identity)))
+  {
+    return NULL;
+  }
 
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Peer `%s' connected to us\n",
               GNUNET_i2s (peer));
+  GNUNET_STATISTICS_update (stats,
+                            "# peers connected",
+                            1,
+                            GNUNET_NO);
   peer_entry = GNUNET_CONTAINER_multipeermap_get (peers,
                                                   peer);
-  if (NULL == peer_entry)
+  if (NULL != peer_entry)
   {
-    peer_entry = GNUNET_new (struct PeerEntry);
-    peer_entry->id = *peer;
-    GNUNET_assert (GNUNET_OK ==
-                   GNUNET_CONTAINER_multipeermap_put (peers, peer,
-                                                      peer_entry,
-                                                      GNUNET_CONTAINER_MULTIHASHMAPOPTION_UNIQUE_ONLY));
+    /* This can happen if "core"'s notification is a tad late
+       and CADET+SET were faster and already produced a
+       #handle_revocation_union_request() for us to deal
+       with.  This should be rare, but isn't impossible. */
+    peer_entry->mq = mq;
+    return peer_entry;
   }
-  else
-  {
-    GNUNET_assert (NULL == peer_entry->mq);
-  }
-  peer_entry->mq = GNUNET_CORE_mq_create (core_api, peer);
-  GNUNET_CRYPTO_hash (&my_identity, sizeof (my_identity), &my_hash);
-  GNUNET_CRYPTO_hash (peer, sizeof (*peer), &peer_hash);
+  peer_entry = new_peer_entry (peer);
+  peer_entry->mq = mq;
+  GNUNET_CRYPTO_hash (&my_identity,
+                      sizeof (my_identity),
+                      &my_hash);
+  GNUNET_CRYPTO_hash (peer,
+                      sizeof (*peer),
+                      &peer_hash);
   if (0 < GNUNET_CRYPTO_hash_cmp (&my_hash,
                                   &peer_hash))
   {
@@ -540,7 +589,7 @@ handle_core_connect (void *cls,
                                     &transmit_task_cb,
                                     peer_entry);
   }
-  GNUNET_STATISTICS_update (stats, "# peers connected", 1, GNUNET_NO);
+  return peer_entry;
 }
 
 
@@ -550,41 +599,42 @@ handle_core_connect (void *cls,
  *
  * @param cls closure
  * @param peer peer identity this notification is about
+ * @param internal_cls our `struct PeerEntry` for this peer
  */
 static void
 handle_core_disconnect (void *cls,
-			const struct GNUNET_PeerIdentity *peer)
+			const struct GNUNET_PeerIdentity *peer,
+			void *internal_cls)
 {
-  struct PeerEntry *pos;
-
-  if (0 == memcmp(peer, &my_identity, sizeof (my_identity)))
-      return;
+  struct PeerEntry *peer_entry = internal_cls;
 
+  if (0 == memcmp (peer,
+                   &my_identity,
+                   sizeof (my_identity)))
+    return;
+  GNUNET_assert (NULL != peer_entry);
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
 	      "Peer `%s' disconnected from us\n",
               GNUNET_i2s (peer));
-  pos = GNUNET_CONTAINER_multipeermap_get (peers, peer);
-  if (NULL == pos)
-  {
-    GNUNET_break (0);
-    return;
-  }
   GNUNET_assert (GNUNET_YES ==
-                 GNUNET_CONTAINER_multipeermap_remove (peers, peer,
-                                                       pos));
-  GNUNET_MQ_destroy (pos->mq);
-  if (pos->transmit_task != GNUNET_SCHEDULER_NO_TASK)
+                 GNUNET_CONTAINER_multipeermap_remove (peers,
+                                                       peer,
+                                                       peer_entry));
+  if (NULL != peer_entry->transmit_task)
   {
-    GNUNET_SCHEDULER_cancel (pos->transmit_task);
-    pos->transmit_task = GNUNET_SCHEDULER_NO_TASK;
+    GNUNET_SCHEDULER_cancel (peer_entry->transmit_task);
+    peer_entry->transmit_task = NULL;
   }
-  if (NULL != pos->so)
+  if (NULL != peer_entry->so)
   {
-    GNUNET_SET_operation_cancel (pos->so);
-    pos->so = NULL;
+    GNUNET_SET_operation_cancel (peer_entry->so);
+    peer_entry->so = NULL;
   }
-  GNUNET_free (pos);
-  GNUNET_STATISTICS_update (stats, "# peers connected", -1, GNUNET_NO);
+  GNUNET_free (peer_entry);
+  GNUNET_STATISTICS_update (stats,
+                            "# peers connected",
+                            -1,
+                            GNUNET_NO);
 }
 
 
@@ -610,11 +660,9 @@ free_entry (void *cls,
  * Task run during shutdown.
  *
  * @param cls unused
- * @param tc unused
  */
 static void
-shutdown_task (void *cls,
-	       const struct GNUNET_SCHEDULER_TaskContext *tc)
+shutdown_task (void *cls)
 {
   if (NULL != revocation_set)
   {
@@ -641,11 +689,6 @@ shutdown_task (void *cls,
     GNUNET_CONTAINER_multipeermap_destroy (peers);
     peers = NULL;
   }
-  if (NULL != nc)
-  {
-    GNUNET_SERVER_notification_context_destroy (nc);
-    nc = NULL;
-  }
   if (NULL != revocation_db)
   {
     GNUNET_DISK_file_close (revocation_db);
@@ -707,21 +750,33 @@ handle_revocation_union_request (void *cls,
     GNUNET_break (0);
     return;
   }
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+              "Received set exchange request from peer `%s'\n",
+              GNUNET_i2s (other_peer));
   peer_entry = GNUNET_CONTAINER_multipeermap_get (peers,
                                                   other_peer);
   if (NULL == peer_entry)
   {
-    peer_entry = GNUNET_new (struct PeerEntry);
-    peer_entry->id = *other_peer;
-    GNUNET_assert (GNUNET_OK ==
-                   GNUNET_CONTAINER_multipeermap_put (peers, other_peer,
-                                                      peer_entry,
-                                                      GNUNET_CONTAINER_MULTIHASHMAPOPTION_UNIQUE_ONLY));
+    peer_entry = new_peer_entry (other_peer);
+  }
+  if (NULL != peer_entry->so)
+  {
+    GNUNET_break_op (0);
+    return;
   }
   peer_entry->so = GNUNET_SET_accept (request,
                                       GNUNET_SET_RESULT_ADDED,
+                                      (struct GNUNET_SET_Option[]) {{ 0 }},
                                       &add_revocation,
                                       peer_entry);
+  if (GNUNET_OK !=
+      GNUNET_SET_commit (peer_entry->so,
+                         revocation_set))
+  {
+    GNUNET_break (0);
+    GNUNET_SCHEDULER_shutdown ();
+    return;
+  }
 }
 
 
@@ -734,26 +789,24 @@ handle_revocation_union_request (void *cls,
  */
 static void
 run (void *cls,
-     struct GNUNET_SERVER_Handle *server,
-     const struct GNUNET_CONFIGURATION_Handle *c)
+     const struct GNUNET_CONFIGURATION_Handle *c,
+     struct GNUNET_SERVICE_Handle *service)
 {
-  static const struct GNUNET_SERVER_MessageHandler handlers[] = {
-    {&handle_query_message, NULL, GNUNET_MESSAGE_TYPE_REVOCATION_QUERY,
-     sizeof (struct QueryMessage)},
-    {&handle_revoke_message, NULL, GNUNET_MESSAGE_TYPE_REVOCATION_REVOKE,
-     sizeof (struct RevokeMessage)},
-    {NULL, NULL, 0, 0}
-  };
-  static const struct GNUNET_CORE_MessageHandler core_handlers[] = {
-    {&handle_p2p_revoke_message, GNUNET_MESSAGE_TYPE_REVOCATION_REVOKE,
-     sizeof (struct RevokeMessage)},
-    {NULL, 0, 0}
+  struct GNUNET_MQ_MessageHandler core_handlers[] = {
+    GNUNET_MQ_hd_fixed_size (p2p_revoke,
+                             GNUNET_MESSAGE_TYPE_REVOCATION_REVOKE,
+                             struct RevokeMessage,
+                             NULL),
+    GNUNET_MQ_handler_end ()
   };
   char *fn;
   uint64_t left;
   struct RevokeMessage *rm;
   struct GNUNET_HashCode hc;
 
+  GNUNET_CRYPTO_hash ("revocation-set-union-application-id",
+                      strlen ("revocation-set-union-application-id"),
+                      &revocation_set_union_app_id);
   if (GNUNET_OK !=
       GNUNET_CONFIGURATION_get_value_filename (c,
                                                "REVOCATION",
@@ -767,11 +820,12 @@ run (void *cls,
     return;
   }
   cfg = c;
-  srv = server;
-  revocation_map = GNUNET_CONTAINER_multihashmap_create (16, GNUNET_NO);
-  nc = GNUNET_SERVER_notification_context_create (server, 1);
+  revocation_map = GNUNET_CONTAINER_multihashmap_create (16,
+							 GNUNET_NO);
   if (GNUNET_OK !=
-      GNUNET_CONFIGURATION_get_value_number (cfg, "REVOCATION", "WORKBITS",
+      GNUNET_CONFIGURATION_get_value_number (cfg,
+					     "REVOCATION",
+					     "WORKBITS",
 					     &revocation_work_required))
   {
     GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
@@ -793,11 +847,12 @@ run (void *cls,
   }
   revocation_set = GNUNET_SET_create (cfg,
 				      GNUNET_SET_OPERATION_UNION);
-  revocation_union_listen_handle = GNUNET_SET_listen (cfg,
-                                                      GNUNET_SET_OPERATION_UNION,
-                                                      &revocation_set_union_app_id,
-                                                      &handle_revocation_union_request,
-                                                      NULL);
+  revocation_union_listen_handle
+    = GNUNET_SET_listen (cfg,
+                         GNUNET_SET_OPERATION_UNION,
+                         &revocation_set_union_app_id,
+                         &handle_revocation_union_request,
+                         NULL);
   revocation_db = GNUNET_DISK_file_open (fn,
                                          GNUNET_DISK_OPEN_READWRITE |
                                          GNUNET_DISK_OPEN_CREATE,
@@ -845,65 +900,63 @@ run (void *cls,
   }
   GNUNET_free (fn);
 
-  GNUNET_SCHEDULER_add_delayed (GNUNET_TIME_UNIT_FOREVER_REL, &shutdown_task,
-                                NULL);
-  peers = GNUNET_CONTAINER_multipeermap_create (128, GNUNET_NO);
-  GNUNET_SERVER_add_handlers (srv, handlers);
-   /* Connect to core service and register core handlers */
+  GNUNET_SCHEDULER_add_shutdown (&shutdown_task,
+				 NULL);
+  peers = GNUNET_CONTAINER_multipeermap_create (128,
+                                                GNUNET_YES);
+  /* Connect to core service and register core handlers */
   core_api = GNUNET_CORE_connect (cfg,   /* Main configuration */
-                                 NULL,       /* Closure passed to functions */
-                                 &core_init,    /* Call core_init once connected */
-                                 &handle_core_connect,  /* Handle connects */
-                                 &handle_core_disconnect,       /* Handle disconnects */
-                                 NULL,  /* Don't want notified about all incoming messages */
-                                 GNUNET_NO,     /* For header only inbound notification */
-                                 NULL,  /* Don't want notified about all outbound messages */
-                                 GNUNET_NO,     /* For header only outbound notification */
-                                 core_handlers);        /* Register these handlers */
+				  NULL,       /* Closure passed to functions */
+				  &core_init,    /* Call core_init once connected */
+				  &handle_core_connect,  /* Handle connects */
+				  &handle_core_disconnect,       /* Handle disconnects */
+				  core_handlers);        /* Register these handlers */
   if (NULL == core_api)
   {
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
-  stats = GNUNET_STATISTICS_create ("revocation", cfg);
+  stats = GNUNET_STATISTICS_create ("revocation",
+				    cfg);
 }
 
 
 /**
- * The main function for the network size estimation service.
- *
- * @param argc number of arguments from the command line
- * @param argv command line arguments
- * @return 0 ok, 1 on error
+ * Define "main" method using service macro.
  */
-int
-main (int argc,
-      char *const *argv)
-{
-  GNUNET_CRYPTO_hash ("revocation-set-union-application-id",
-                      strlen ("revocation-set-union-application-id"),
-                      &revocation_set_union_app_id);
-  return (GNUNET_OK ==
-          GNUNET_SERVICE_run (argc, argv, "revocation", GNUNET_SERVICE_OPTION_NONE,
-                              &run, NULL)) ? 0 : 1;
-}
+GNUNET_SERVICE_MAIN
+("revocation",
+ GNUNET_SERVICE_OPTION_NONE,
+ &run,
+ &client_connect_cb,
+ &client_disconnect_cb,
+ NULL,
+ GNUNET_MQ_hd_fixed_size (query_message,
+			  GNUNET_MESSAGE_TYPE_REVOCATION_QUERY,
+			  struct QueryMessage,
+			  NULL),
+ GNUNET_MQ_hd_fixed_size (revoke_message,
+			  GNUNET_MESSAGE_TYPE_REVOCATION_REVOKE,
+			  struct RevokeMessage,
+			  NULL),
+ GNUNET_MQ_handler_end ());
 
 
-#ifdef LINUX
+#if defined(LINUX) && defined(__GLIBC__)
 #include <malloc.h>
 
 /**
  * MINIMIZE heap size (way below 128k) since this process doesn't need much.
  */
 void __attribute__ ((constructor))
-GNUNET_ARM_memory_init ()
+GNUNET_REVOCATION_memory_init ()
 {
   mallopt (M_TRIM_THRESHOLD, 4 * 1024);
   mallopt (M_TOP_PAD, 1 * 1024);
   malloc_trim (0);
 }
+
 #endif
 
 
-
 /* end of gnunet-service-revocation.c */
