@@ -1,34 +1,33 @@
 /*
      This file is part of GNUnet.
-     (C) 2010,2011 Christian Grothoff (and other contributing authors)
+     Copyright (C) 2010-2015 GNUnet e.V.
 
-     GNUnet is free software; you can redistribute it and/or modify
-     it under the terms of the GNU General Public License as published
-     by the Free Software Foundation; either version 3, or (at your
-     option) any later version.
+     GNUnet is free software: you can redistribute it and/or modify it
+     under the terms of the GNU Affero General Public License as published
+     by the Free Software Foundation, either version 3 of the License,
+     or (at your option) any later version.
 
      GNUnet is distributed in the hope that it will be useful, but
      WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-     General Public License for more details.
+     Affero General Public License for more details.
+    
+     You should have received a copy of the GNU Affero General Public License
+     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-     You should have received a copy of the GNU General Public License
-     along with GNUnet; see the file COPYING.  If not, write to the
-     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-     Boston, MA 02111-1307, USA.
+     SPDX-License-Identifier: AGPL3.0-or-later
 */
 
 /**
  * @file transport/gnunet-service-transport_neighbours.h
- * @brief neighbour management API
+ * @brief neighbour manipulation API, allows manipulation of
+ *        performance metrics (delay and towards ATS)
  * @author Christian Grothoff
  */
 #ifndef GNUNET_SERVICE_TRANSPORT_MANIPULATION_H
 #define GNUNET_SERVICE_TRANSPORT_MANIPULATION_H
 
 #include "platform.h"
-#include "gnunet-service-transport_blacklist.h"
-#include "gnunet-service-transport_clients.h"
 #include "gnunet-service-transport_hello.h"
 #include "gnunet-service-transport_neighbours.h"
 #include "gnunet-service-transport_plugins.h"
@@ -40,14 +39,11 @@
 /**
  * Set traffic metric to manipulate
  *
- * @param cls closure
- * @param client client sending message
  * @param message containing information
  */
-
 void
-GST_manipulation_set_metric (void *cls, struct GNUNET_SERVER_Client *client,
-    const struct GNUNET_MessageHeader *message);
+GST_manipulation_set_metric (const struct TrafficMetricMessage *tm);
+
 
 /**
  * Adapter function between transport's send function and transport plugins
@@ -61,9 +57,12 @@ GST_manipulation_set_metric (void *cls, struct GNUNET_SERVER_Client *client,
  */
 void
 GST_manipulation_send (const struct GNUNET_PeerIdentity *target,
-											 const void *msg, size_t msg_size,
-											 struct GNUNET_TIME_Relative timeout,
-											 GST_NeighbourSendContinuation cont, void *cont_cls);
+                       const void *msg,
+                       size_t msg_size,
+                       struct GNUNET_TIME_Relative timeout,
+                       GST_NeighbourSendContinuation cont,
+                       void *cont_cls);
+
 
 /**
  * Adapter function between transport plugins and transport receive function
@@ -77,26 +76,24 @@ GST_manipulation_send (const struct GNUNET_PeerIdentity *target,
  */
 struct GNUNET_TIME_Relative
 GST_manipulation_recv (void *cls,
-    const struct GNUNET_HELLO_Address *address,
-    struct Session *session,
-    const struct GNUNET_MessageHeader *message);
+                       const struct GNUNET_HELLO_Address *address,
+                       struct GNUNET_ATS_Session *session,
+                       const struct GNUNET_MessageHeader *message);
+
 
 /**
  * Function that will be called to manipulate ATS information according to
  * current manipulation settings
  *
- * @param peer the peer
  * @param address binary address
  * @param session the session
- * @param ats the ats information
- * @param ats_count the number of ats information
+ * @param prop[IN|OUT] metrics to modify
  */
-struct GNUNET_ATS_Information *
-GST_manipulation_manipulate_metrics (const struct GNUNET_PeerIdentity *peer,
-		const struct GNUNET_HELLO_Address *address,
-		struct Session *session,
-		const struct GNUNET_ATS_Information *ats,
-		uint32_t ats_count);
+void
+GST_manipulation_manipulate_metrics (const struct GNUNET_HELLO_Address *address,
+                                     struct GNUNET_ATS_Session *session,
+                                     struct GNUNET_ATS_Properties *prop);
+
 
 /**
  * Notify manipulation about disconnect so it can discard queued messages
@@ -106,19 +103,19 @@ GST_manipulation_manipulate_metrics (const struct GNUNET_PeerIdentity *peer,
 void
 GST_manipulation_peer_disconnect (const struct GNUNET_PeerIdentity *peer);
 
+
 /**
  * Initialize traffic manipulation
- *
- * @param GST_cfg configuration handle
  */
 void
-GST_manipulation_init (const struct GNUNET_CONFIGURATION_Handle *GST_cfg);
+GST_manipulation_init (void);
+
 
 /**
  * Stop traffic manipulation
  */
 void
-GST_manipulation_stop ();
+GST_manipulation_stop (void);
 
 #endif
 /* end of file gnunet-service-transport_neighbours.h */
