@@ -1,21 +1,21 @@
 /*
      This file is part of GNUnet.
-     (C) 2010,2011 Christian Grothoff (and other contributing authors)
+     Copyright (C) 2010-2015 GNUnet e.V.
 
-     GNUnet is free software; you can redistribute it and/or modify
-     it under the terms of the GNU General Public License as published
-     by the Free Software Foundation; either version 3, or (at your
-     option) any later version.
+     GNUnet is free software: you can redistribute it and/or modify it
+     under the terms of the GNU Affero General Public License as published
+     by the Free Software Foundation, either version 3 of the License,
+     or (at your option) any later version.
 
      GNUnet is distributed in the hope that it will be useful, but
      WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-     General Public License for more details.
+     Affero General Public License for more details.
+    
+     You should have received a copy of the GNU Affero General Public License
+     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-     You should have received a copy of the GNU General Public License
-     along with GNUnet; see the file COPYING.  If not, write to the
-     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-     Boston, MA 02111-1307, USA.
+     SPDX-License-Identifier: AGPL3.0-or-later
 */
 
 /**
@@ -33,26 +33,14 @@
 #include "transport.h"
 #include "gnunet_util_lib.h"
 
-// TODO:
-// - ATS and similar info is a bit lacking in the API right now...
-
-
 
 /**
  * Initialize the neighbours subsystem.
  *
- * @param cls closure for callbacks
- * @param connect_cb function to call if we connect to a peer
- * @param disconnect_cb function to call if we disconnect from a peer
- * @param peer_address_cb function to call if a neighbour's active address changes
  * @param max_fds maximum number of fds to use
  */
 void
-GST_neighbours_start (void *cls,
-                      NotifyConnect connect_cb,
-                      GNUNET_TRANSPORT_NotifyDisconnect disconnect_cb,
-                      GNUNET_TRANSPORT_NeighbourChangeCallback peer_address_cb,
-                      unsigned int max_fds);
+GST_neighbours_start (unsigned int max_fds);
 
 
 /**
@@ -62,15 +50,6 @@ void
 GST_neighbours_stop (void);
 
 
-/**
- * Try to create a connection to the given target (eventually).
- *
- * @param target peer to try to connect to
- */
-void
-GST_neighbours_try_connect (const struct GNUNET_PeerIdentity *target);
-
-
 /**
  * Test if we're connected to the given peer.
  *
@@ -86,10 +65,14 @@ GST_neighbours_test_connected (const struct GNUNET_PeerIdentity *target);
  *
  * @param cls closure
  * @param success #GNUNET_OK on success, #GNUNET_NO on failure, #GNUNET_SYSERR if we're not connected
+ * @param bytes_payload how much payload was transmitted
+ * @param bytes_on_wire how many bytes were used on the wire
  */
-typedef void (*GST_NeighbourSendContinuation) (void *cls, int success,
-                                               size_t bytes_payload,
-                                               size_t bytes_on_wire);
+typedef void
+(*GST_NeighbourSendContinuation) (void *cls,
+                                  int success,
+                                  size_t bytes_payload,
+                                  size_t bytes_on_wire);
 
 
 /**
@@ -103,19 +86,12 @@ typedef void (*GST_NeighbourSendContinuation) (void *cls, int success,
  * @param cont_cls closure for @a cont
  */
 void
-GST_neighbours_send (const struct GNUNET_PeerIdentity *target, const void *msg,
-                     size_t msg_size, struct GNUNET_TIME_Relative timeout,
+GST_neighbours_send (const struct GNUNET_PeerIdentity *target,
+                     const void *msg,
+                     size_t msg_size,
+                     struct GNUNET_TIME_Relative timeout,
                      GST_NeighbourSendContinuation cont, void *cont_cls);
 
-void
-GST_neighbours_register_quota_notification (void *cls,
-                                           const struct GNUNET_PeerIdentity *peer,
-                                           const char *plugin,
-                                           struct Session *session);
-
-void
-GST_neighbours_unregister_quota_notification(void *cls,
-    const struct GNUNET_PeerIdentity *peer, const char *plugin, struct Session *session);
 
 /**
  * We have received a message from the given sender.
@@ -129,8 +105,9 @@ GST_neighbours_unregister_quota_notification(void *cls,
  * @return how long to wait before reading more from this sender
  */
 struct GNUNET_TIME_Relative
-GST_neighbours_calculate_receive_delay (const struct GNUNET_PeerIdentity
-                                        *sender, ssize_t size, int *do_forward);
+GST_neighbours_calculate_receive_delay (const struct GNUNET_PeerIdentity *sender,
+                                        ssize_t size,
+                                        int *do_forward);
 
 
 /**
@@ -155,18 +132,7 @@ GST_neighbours_keepalive (const struct GNUNET_PeerIdentity *neighbour,
  */
 void
 GST_neighbours_keepalive_response (const struct GNUNET_PeerIdentity *neighbour,
-    const struct GNUNET_MessageHeader *m);
-
-
-/**
- * Change the incoming quota for the given peer.
- *
- * @param neighbour identity of peer to change qutoa for
- * @param quota new quota
- */
-void
-GST_neighbours_set_incoming_quota (const struct GNUNET_PeerIdentity *neighbour,
-                                   struct GNUNET_BANDWIDTH_Value32NBO quota);
+                                   const struct GNUNET_MessageHeader *m);
 
 
 /**
@@ -182,20 +148,21 @@ GST_neighbours_force_disconnect (const struct GNUNET_PeerIdentity *target);
  * Function called for each neighbour.
  *
  * @param cls closure
- * @param neighbour identity of the neighbour
- * @param address the address (or NULL)
+ * @param peer identity of the neighbour
+ * @param address the address of the neighbour
  * @param state current state the peer is in
  * @param state_timeout timeout for this state
  * @param bandwidth_in inbound quota in NBO
  * @param bandwidth_out outbound quota in NBO
  */
-typedef void (*GST_NeighbourIterator) (void *cls,
-                                       const struct GNUNET_PeerIdentity *neighbour,
-                                       const struct GNUNET_HELLO_Address *address,
-                                       enum GNUNET_TRANSPORT_PeerState state,
-                                       struct GNUNET_TIME_Absolute state_timeout,
-                                       struct GNUNET_BANDWIDTH_Value32NBO bandwidth_in,
-                                       struct GNUNET_BANDWIDTH_Value32NBO bandwidth_out);
+typedef void
+(*GST_NeighbourIterator) (void *cls,
+                          const struct GNUNET_PeerIdentity *peer,
+                          const struct GNUNET_HELLO_Address *address,
+                          enum GNUNET_TRANSPORT_PeerState state,
+                          struct GNUNET_TIME_Absolute state_timeout,
+                          struct GNUNET_BANDWIDTH_Value32NBO bandwidth_in,
+                          struct GNUNET_BANDWIDTH_Value32NBO bandwidth_out);
 
 
 /**
@@ -218,32 +185,34 @@ GST_neighbours_iterate (GST_NeighbourIterator cb, void *cb_cls);
  */
 int
 GST_neighbours_session_terminated (const struct GNUNET_PeerIdentity *peer,
-                                   struct Session *session);
+                                   struct GNUNET_ATS_Session *session);
 
 
+/**
+ * Track information about data we received from the
+ * given address (used to notify ATS about our utilization
+ * of allocated resources).
+ *
+ * @param address the address we got data from
+ * @param message the message we received (really only the size is used)
+ */
 void
-GST_neighbours_notify_data_recv (const struct GNUNET_PeerIdentity *peer,
-                                 const struct GNUNET_HELLO_Address *address,
-                                 struct Session *session,
+GST_neighbours_notify_data_recv (const struct GNUNET_HELLO_Address *address,
                                  const struct GNUNET_MessageHeader *message);
 
 
+/**
+ * Track information about data we transmitted using the given @a
+ * address and @a session (used to notify ATS about our utilization of
+ * allocated resources).
+ *
+ * @param address the address we transmitted data to
+ * @param session session we used to transmit data
+ * @param message the message we sent (really only the size is used)
+ */
 void
-GST_neighbours_notify_payload_recv (const struct GNUNET_PeerIdentity *peer,
-                                    const struct GNUNET_HELLO_Address *address,
-                                    struct Session *session,
-                                    const struct GNUNET_MessageHeader *message);
-
-
-void
-GST_neighbours_notify_payload_sent (const struct GNUNET_PeerIdentity *peer,
-                                    size_t size);
-
-
-void
-GST_neighbours_notify_data_sent (const struct GNUNET_PeerIdentity *peer,
-                                 const struct GNUNET_HELLO_Address *address,
-                                 struct Session *session,
+GST_neighbours_notify_data_sent (const struct GNUNET_HELLO_Address *address,
+                                 struct GNUNET_ATS_Session *session,
                                  size_t size);
 
 
@@ -251,21 +220,14 @@ GST_neighbours_notify_data_sent (const struct GNUNET_PeerIdentity *peer,
  * For an existing neighbour record, set the active connection to
  * use the given address.
  *
- * @param peer identity of the peer to switch the address for
- * @param address address of the other peer, NULL if other peer
- *                       connected to us
+ * @param address address of the other peer to start using
  * @param session session to use (or NULL)
- * @param ats performance data
- * @param ats_count number of entries in ats
  * @param bandwidth_in inbound quota to be used when connection is up
  * @param bandwidth_out outbound quota to be used when connection is up
  */
 void
-GST_neighbours_switch_to_address (const struct GNUNET_PeerIdentity *peer,
-                                  const struct GNUNET_HELLO_Address *address,
-                                  struct Session *session,
-                                  const struct GNUNET_ATS_Information *ats,
-                                  uint32_t ats_count,
+GST_neighbours_switch_to_address (const struct GNUNET_HELLO_Address *address,
+                                  struct GNUNET_ATS_Session *session,
                                   struct GNUNET_BANDWIDTH_Value32NBO bandwidth_in,
                                   struct GNUNET_BANDWIDTH_Value32NBO bandwidth_out);
 
@@ -274,31 +236,28 @@ GST_neighbours_switch_to_address (const struct GNUNET_PeerIdentity *peer,
  * We received a 'SESSION_CONNECT' message from the other peer.
  * Consider switching to it.
  *
- * @param message possibly a 'struct SessionConnectMessage' (check format)
+ * @param message possibly a 'struct GNUNET_ATS_SessionConnectMessage' (check format)
  * @param peer identity of the peer to switch the address for
  * @return #GNUNET_OK if the message was fine, #GNUNET_SYSERR on serious error
  */
 int
-GST_neighbours_handle_connect (const struct GNUNET_MessageHeader *message,
-                               const struct GNUNET_PeerIdentity *peer);
+GST_neighbours_handle_session_syn (const struct GNUNET_MessageHeader *message,
+                                   const struct GNUNET_PeerIdentity *peer);
 
 
 /**
  * We received a 'SESSION_CONNECT_ACK' message from the other peer.
  * Consider switching to it.
  *
- * @param message possibly a 'struct SessionConnectMessage' (check format)
- * @param peer identity of the peer to switch the address for
- * @param address address of the other peer, NULL if other peer
- *                       connected to us
+ * @param message possibly a `struct GNUNET_ATS_SessionConnectMessage` (check format)
+ * @param address address of the other peer
  * @param session session to use (or NULL)
  * @return #GNUNET_OK if the message was fine, #GNUNET_SYSERR on serious error
  */
 int
-GST_neighbours_handle_connect_ack (const struct GNUNET_MessageHeader *message,
-                                   const struct GNUNET_PeerIdentity *peer,
-                                   const struct GNUNET_HELLO_Address *address,
-                                   struct Session *session);
+GST_neighbours_handle_session_syn_ack (const struct GNUNET_MessageHeader *message,
+                                       const struct GNUNET_HELLO_Address *address,
+                                       struct GNUNET_ATS_Session *session);
 
 
 /**
@@ -306,29 +265,15 @@ GST_neighbours_handle_connect_ack (const struct GNUNET_MessageHeader *message,
  * If we sent a 'CONNECT_ACK' last, this means we are now
  * connected.  Otherwise, do nothing.
  *
- * @param message possibly a 'struct SessionConnectMessage' (check format)
- * @param peer identity of the peer to switch the address for
- * @param address address of the other peer, NULL if other peer
- *                       connected to us
+ * @param message possibly a 'struct GNUNET_ATS_SessionConnectMessage' (check format)
+ * @param address address of the other peer
  * @param session session to use (or NULL)
  * @return #GNUNET_OK if the message was fine, #GNUNET_SYSERR on serious error
  */
 int
 GST_neighbours_handle_session_ack (const struct GNUNET_MessageHeader *message,
-				   const struct GNUNET_PeerIdentity *peer,
 				   const struct GNUNET_HELLO_Address *address,
-				   struct Session *session);
-
-
-/**
- * Obtain current latency information for the given neighbour.
- *
- * @param peer
- * @return observed latency of the address, FOREVER if the address was
- *         never successfully validated
- */
-struct GNUNET_TIME_Relative
-GST_neighbour_get_latency (const struct GNUNET_PeerIdentity *peer);
+				   struct GNUNET_ATS_Session *session);
 
 
 /**
@@ -337,10 +282,22 @@ GST_neighbour_get_latency (const struct GNUNET_PeerIdentity *peer);
  * @param peer
  * @return address currently used
  */
-struct GNUNET_HELLO_Address *
+const struct GNUNET_HELLO_Address *
 GST_neighbour_get_current_address (const struct GNUNET_PeerIdentity *peer);
 
 
+/**
+ * We received a quoat message from the given peer,
+ * validate and process.
+ *
+ * @param peer sender of the message
+ * @param msg the quota message
+ */
+void
+GST_neighbours_handle_quota_message (const struct GNUNET_PeerIdentity *peer,
+                                     const struct GNUNET_MessageHeader *msg);
+
+
 /**
  * We received a disconnect message from the given peer,
  * validate and process.
@@ -349,10 +306,8 @@ GST_neighbour_get_current_address (const struct GNUNET_PeerIdentity *peer);
  * @param msg the disconnect message
  */
 void
-GST_neighbours_handle_disconnect_message (const struct GNUNET_PeerIdentity
-                                          *peer,
-                                          const struct GNUNET_MessageHeader
-                                          *msg);
+GST_neighbours_handle_disconnect_message (const struct GNUNET_PeerIdentity *peer,
+                                          const struct GNUNET_MessageHeader *msg);
 
 
 #endif
