@@ -1,21 +1,21 @@
 /*
      This file is part of GNUnet.
-     (C) 2008, 2012 Christian Grothoff (and other contributing authors)
+     Copyright (C) 2008, 2012 GNUnet e.V.
 
-     GNUnet is free software; you can redistribute it and/or modify
-     it under the terms of the GNU General Public License as published
-     by the Free Software Foundation; either version 3, or (at your
-     option) any later version.
+     GNUnet is free software: you can redistribute it and/or modify it
+     under the terms of the GNU Affero General Public License as published
+     by the Free Software Foundation, either version 3 of the License,
+     or (at your option) any later version.
 
      GNUnet is distributed in the hope that it will be useful, but
      WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-     General Public License for more details.
+     Affero General Public License for more details.
 
-     You should have received a copy of the GNU General Public License
-     along with GNUnet; see the file COPYING.  If not, write to the
-     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-     Boston, MA 02111-1307, USA.
+     You should have received a copy of the GNU Affero General Public License
+     along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+     SPDX-License-Identifier: AGPL3.0-or-later
 */
 /**
  * @file util/container_multihashmap.c
@@ -24,9 +24,18 @@
  */
 
 #include "platform.h"
-#include "gnunet_util_lib.h"
+#include "gnunet_container_lib.h"
+
+#define LOG(kind,...) GNUNET_log_from (kind, "util-container-multihashmap", __VA_ARGS__)
+
+/**
+ * Maximum recursion depth for callbacks of
+ * #GNUNET_CONTAINER_multihashmap_get_multiple() themselve s
+ * again calling #GNUNET_CONTAINER_multihashmap_get_multiple().
+ * Should be totally excessive, but if violated we die.
+ */
+#define NEXT_CACHE_SIZE 16
 
-#define LOG(kind,...) GNUNET_log_from (kind, "util", __VA_ARGS__)
 
 /**
  * An entry in the hash map with the full key.
@@ -124,6 +133,19 @@ struct GNUNET_CONTAINER_MultiHashMap
    * to the map, so that iterators can check if they are still valid.
    */
   unsigned int modification_counter;
+
+  /**
+   * Map entries indicating iteration positions currently
+   * in use by #GNUNET_CONTAINER_multihashmap_get_multiple().
+   * Only used up to @e next_cache_off.
+   */
+  union MapEntry next_cache[NEXT_CACHE_SIZE];
+
+  /**
+   * Offset of @e next_cache entries in use, must be smaller
+   * than #NEXT_CACHE_SIZE.
+   */
+  unsigned int next_cache_off;
 };
 
 
@@ -134,7 +156,7 @@ struct GNUNET_CONTAINER_MultiHashMap
 struct GNUNET_CONTAINER_MultiHashMapIterator
 {
   /**
-   * Position in the bucket 'idx'
+   * Position in the bucket @e idx
    */
   union MapEntry me;
 
@@ -175,32 +197,53 @@ struct GNUNET_CONTAINER_MultiHashMap *
 GNUNET_CONTAINER_multihashmap_create (unsigned int len,
 				      int do_not_copy_keys)
 {
-  struct GNUNET_CONTAINER_MultiHashMap *map;
+  struct GNUNET_CONTAINER_MultiHashMap *hm;
 
   GNUNET_assert (len > 0);
-  map = GNUNET_new (struct GNUNET_CONTAINER_MultiHashMap);
-  map->map = GNUNET_malloc (len * sizeof (union MapEntry));
-  map->map_length = len;
-  map->use_small_entries = do_not_copy_keys;
-  return map;
+  hm = GNUNET_new (struct GNUNET_CONTAINER_MultiHashMap);
+  if (len * sizeof (union MapEntry) > GNUNET_MAX_MALLOC_CHECKED)
+  {
+    size_t s;
+    /* application *explicitly* requested very large map, hopefully
+       it checks the return value... */
+    s = len * sizeof (union MapEntry);
+    if ( (s / sizeof (union MapEntry)) != len)
+      return NULL; /* integer overflow on multiplication */
+    if (NULL == (hm->map = GNUNET_malloc_large (s)))
+    {
+      /* out of memory */
+      GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                  "Out of memory allocating large hash map (%u entries)\n",
+                  len);
+      GNUNET_free (hm);
+      return NULL;
+    }
+  }
+  else
+  {
+    hm->map = GNUNET_new_array (len,
+                                union MapEntry);
+  }
+  hm->map_length = len;
+  hm->use_small_entries = do_not_copy_keys;
+  return hm;
 }
 
 
 /**
- * Destroy a hash map.  Will not free any values
- * stored in the hash map!
+ * Destroy a hash map.  Will not free any values stored in the hash
+ * map!
  *
  * @param map the map
  */
 void
-GNUNET_CONTAINER_multihashmap_destroy (struct GNUNET_CONTAINER_MultiHashMap
-                                       *map)
+GNUNET_CONTAINER_multihashmap_destroy (struct GNUNET_CONTAINER_MultiHashMap *map)
 {
-  unsigned int i;
-  union MapEntry me;
-
-  for (i = 0; i < map->map_length; i++)
+  GNUNET_assert (0 == map->next_cache_off);
+  for (unsigned int i = 0; i < map->map_length; i++)
   {
+    union MapEntry me;
+
     me = map->map[i];
     if (map->use_small_entries)
     {
@@ -257,8 +300,7 @@ idx_of (const struct GNUNET_CONTAINER_MultiHashMap *map,
  * @return the number of key value pairs
  */
 unsigned int
-GNUNET_CONTAINER_multihashmap_size (const struct GNUNET_CONTAINER_MultiHashMap
-                                    *map)
+GNUNET_CONTAINER_multihashmap_size (const struct GNUNET_CONTAINER_MultiHashMap *map)
 {
   return map->size;
 }
@@ -275,8 +317,8 @@ GNUNET_CONTAINER_multihashmap_size (const struct GNUNET_CONTAINER_MultiHashMap
  *   key-value pairs with value NULL
  */
 void *
-GNUNET_CONTAINER_multihashmap_get (const struct GNUNET_CONTAINER_MultiHashMap
-                                   *map, const struct GNUNET_HashCode *key)
+GNUNET_CONTAINER_multihashmap_get (const struct GNUNET_CONTAINER_MultiHashMap *map,
+                                   const struct GNUNET_HashCode *key)
 {
   union MapEntry me;
 
@@ -286,7 +328,9 @@ GNUNET_CONTAINER_multihashmap_get (const struct GNUNET_CONTAINER_MultiHashMap
     struct SmallMapEntry *sme;
 
     for (sme = me.sme; NULL != sme; sme = sme->next)
-      if (0 == memcmp (key, sme->key, sizeof (struct GNUNET_HashCode)))
+      if (0 == memcmp (key,
+		       sme->key,
+		       sizeof (struct GNUNET_HashCode)))
 	return sme->value;
   }
   else
@@ -294,7 +338,9 @@ GNUNET_CONTAINER_multihashmap_get (const struct GNUNET_CONTAINER_MultiHashMap
     struct BigMapEntry *bme;
 
     for (bme = me.bme; NULL != bme; bme = bme->next)
-      if (0 == memcmp (key, &bme->key, sizeof (struct GNUNET_HashCode)))
+      if (0 == memcmp (key,
+		       &bme->key,
+		       sizeof (struct GNUNET_HashCode)))
 	return bme->value;
   }
   return NULL;
@@ -311,34 +357,39 @@ GNUNET_CONTAINER_multihashmap_get (const struct GNUNET_CONTAINER_MultiHashMap
  *         #GNUNET_SYSERR if it aborted iteration
  */
 int
-GNUNET_CONTAINER_multihashmap_iterate (const struct
-                                       GNUNET_CONTAINER_MultiHashMap *map,
+GNUNET_CONTAINER_multihashmap_iterate (struct GNUNET_CONTAINER_MultiHashMap *map,
                                        GNUNET_CONTAINER_HashMapIterator it,
                                        void *it_cls)
 {
   int count;
-  unsigned int i;
   union MapEntry me;
+  union MapEntry *ce;
   struct GNUNET_HashCode kc;
 
-  count = 0;
   GNUNET_assert (NULL != map);
-  for (i = 0; i < map->map_length; i++)
+  ce = &map->next_cache[map->next_cache_off];
+  GNUNET_assert (++map->next_cache_off < NEXT_CACHE_SIZE);
+  count = 0;
+  for (unsigned i = 0; i < map->map_length; i++)
   {
     me = map->map[i];
     if (map->use_small_entries)
     {
       struct SmallMapEntry *sme;
-      struct SmallMapEntry *nxt;
 
-      nxt = me.sme;
-      while (NULL != (sme = nxt))
+      ce->sme = me.sme;
+      while (NULL != (sme = ce->sme))
       {
-	nxt = sme->next;
+	ce->sme = sme->next;
 	if (NULL != it)
 	{
-	  if (GNUNET_OK != it (it_cls, sme->key, sme->value))
+	  if (GNUNET_OK != it (it_cls,
+			       sme->key,
+			       sme->value))
+	  {
+	    GNUNET_assert (--map->next_cache_off < NEXT_CACHE_SIZE);
 	    return GNUNET_SYSERR;
+	  }
 	}
 	count++;
       }
@@ -346,26 +397,65 @@ GNUNET_CONTAINER_multihashmap_iterate (const struct
     else
     {
       struct BigMapEntry *bme;
-      struct BigMapEntry *nxt;
 
-      nxt = me.bme;
-      while (NULL != (bme = nxt))
+      ce->bme = me.bme;
+      while (NULL != (bme = ce->bme))
       {
-	nxt = bme->next;
+	ce->bme = bme->next;
 	if (NULL != it)
 	{
 	  kc = bme->key;
-	  if (GNUNET_OK != it (it_cls, &kc, bme->value))
+	  if (GNUNET_OK != it (it_cls,
+			       &kc,
+			       bme->value))
+	  {
+	    GNUNET_assert (--map->next_cache_off < NEXT_CACHE_SIZE);
 	    return GNUNET_SYSERR;
+	  }
 	}
 	count++;
       }
     }
   }
+  GNUNET_assert (--map->next_cache_off < NEXT_CACHE_SIZE);
   return count;
 }
 
 
+/**
+ * We are about to free() the @a bme, make sure it is not in
+ * the list of next values for any iterator in the @a map's next_cache.
+ *
+ * @param map the map to check
+ * @param bme the entry that is about to be free'd
+ */
+static void
+update_next_cache_bme (struct GNUNET_CONTAINER_MultiHashMap *map,
+		       const struct BigMapEntry *bme)
+{
+  for (unsigned int i=0;i<map->next_cache_off;i++)
+    if (map->next_cache[i].bme == bme)
+      map->next_cache[i].bme = bme->next;
+}
+
+
+/**
+ * We are about to free() the @a sme, make sure it is not in
+ * the list of next values for any iterator in the @a map's next_cache.
+ *
+ * @param map the map to check
+ * @param sme the entry that is about to be free'd
+ */
+static void
+update_next_cache_sme (struct GNUNET_CONTAINER_MultiHashMap *map,
+		       const struct SmallMapEntry *sme)
+{
+  for (unsigned int i=0;i<map->next_cache_off;i++)
+    if (map->next_cache[i].sme == sme)
+      map->next_cache[i].sme = sme->next;
+}
+
+
 /**
  * Remove the given key-value pair from the map.  Note that if the
  * key-value pair is in the map multiple times, only one of the pairs
@@ -391,19 +481,22 @@ GNUNET_CONTAINER_multihashmap_remove (struct GNUNET_CONTAINER_MultiHashMap *map,
   me = map->map[i];
   if (map->use_small_entries)
   {
-    struct SmallMapEntry *sme;
     struct SmallMapEntry *p;
 
     p = NULL;
-    for (sme = me.sme; NULL != sme; sme = sme->next)
+    for (struct SmallMapEntry *sme = me.sme; NULL != sme; sme = sme->next)
     {
-      if ((0 == memcmp (key, sme->key, sizeof (struct GNUNET_HashCode))) &&
-	  (value == sme->value))
+      if ( (0 == memcmp (key,
+			 sme->key,
+			 sizeof (struct GNUNET_HashCode))) &&
+	   (value == sme->value) )
       {
 	if (NULL == p)
 	  map->map[i].sme = sme->next;
 	else
 	  p->next = sme->next;
+	update_next_cache_sme (map,
+			       sme);
 	GNUNET_free (sme);
 	map->size--;
 	return GNUNET_YES;
@@ -413,19 +506,22 @@ GNUNET_CONTAINER_multihashmap_remove (struct GNUNET_CONTAINER_MultiHashMap *map,
   }
   else
   {
-    struct BigMapEntry *bme;
     struct BigMapEntry *p;
 
     p = NULL;
-    for (bme = me.bme; NULL != bme; bme = bme->next)
+    for (struct BigMapEntry *bme = me.bme; NULL != bme; bme = bme->next)
     {
-      if ((0 == memcmp (key, &bme->key, sizeof (struct GNUNET_HashCode))) &&
-	  (value == bme->value))
+      if ( (0 == memcmp (key,
+			 &bme->key,
+			 sizeof (struct GNUNET_HashCode))) &&
+	   (value == bme->value) )
       {
 	if (NULL == p)
 	  map->map[i].bme = bme->next;
 	else
 	  p->next = bme->next;
+	update_next_cache_bme (map,
+			       bme);
 	GNUNET_free (bme);
 	map->size--;
 	return GNUNET_YES;
@@ -446,8 +542,8 @@ GNUNET_CONTAINER_multihashmap_remove (struct GNUNET_CONTAINER_MultiHashMap *map,
  * @return number of values removed
  */
 int
-GNUNET_CONTAINER_multihashmap_remove_all (struct GNUNET_CONTAINER_MultiHashMap
-                                          *map, const struct GNUNET_HashCode *key)
+GNUNET_CONTAINER_multihashmap_remove_all (struct GNUNET_CONTAINER_MultiHashMap *map,
+                                          const struct GNUNET_HashCode *key)
 {
   union MapEntry me;
   unsigned int i;
@@ -467,12 +563,16 @@ GNUNET_CONTAINER_multihashmap_remove_all (struct GNUNET_CONTAINER_MultiHashMap
     sme = me.sme;
     while (NULL != sme)
     {
-      if (0 == memcmp (key, sme->key, sizeof (struct GNUNET_HashCode)))
+      if (0 == memcmp (key,
+		       sme->key,
+		       sizeof (struct GNUNET_HashCode)))
       {
 	if (NULL == p)
 	  map->map[i].sme = sme->next;
 	else
 	  p->next = sme->next;
+	update_next_cache_sme (map,
+			       sme);
 	GNUNET_free (sme);
 	map->size--;
 	if (NULL == p)
@@ -497,12 +597,16 @@ GNUNET_CONTAINER_multihashmap_remove_all (struct GNUNET_CONTAINER_MultiHashMap
     bme = me.bme;
     while (NULL != bme)
     {
-      if (0 == memcmp (key, &bme->key, sizeof (struct GNUNET_HashCode)))
+      if (0 == memcmp (key,
+		       &bme->key,
+		       sizeof (struct GNUNET_HashCode)))
       {
 	if (NULL == p)
 	  map->map[i].bme = bme->next;
 	else
 	  p->next = bme->next;
+	update_next_cache_bme (map,
+			       bme);
 	GNUNET_free (bme);
 	map->size--;
 	if (NULL == p)
@@ -522,6 +626,49 @@ GNUNET_CONTAINER_multihashmap_remove_all (struct GNUNET_CONTAINER_MultiHashMap
 }
 
 
+/**
+ * Callback used to remove all entries from the map.
+ *
+ * @param cls the `struct GNUNET_CONTAINER_MultiHashMap`
+ * @param key the key
+ * @param value the value
+ * @return #GNUNET_OK (continue to iterate)
+ */
+static int
+remove_all (void *cls,
+            const struct GNUNET_HashCode *key,
+            void *value)
+{
+  struct GNUNET_CONTAINER_MultiHashMap *map = cls;
+
+  GNUNET_CONTAINER_multihashmap_remove (map,
+                                        key,
+                                        value);
+  return GNUNET_OK;
+}
+
+
+/**
+ * @ingroup hashmap
+ * Remove all entries from the map.
+ * Note that the values would not be "freed".
+ *
+ * @param map the map
+ * @return number of values removed
+ */
+unsigned int
+GNUNET_CONTAINER_multihashmap_clear (struct GNUNET_CONTAINER_MultiHashMap *map)
+{
+  unsigned int ret;
+
+  ret = map->size;
+  GNUNET_CONTAINER_multihashmap_iterate (map,
+                                         &remove_all,
+                                         map);
+  return ret;
+}
+
+
 /**
  * Check if the map contains any value under the given
  * key (including values that are NULL).
@@ -570,9 +717,8 @@ GNUNET_CONTAINER_multihashmap_contains (const struct
  *         #GNUNET_NO if not
  */
 int
-GNUNET_CONTAINER_multihashmap_contains_value (const struct
-                                              GNUNET_CONTAINER_MultiHashMap
-                                              *map, const struct GNUNET_HashCode *key,
+GNUNET_CONTAINER_multihashmap_contains_value (const struct GNUNET_CONTAINER_MultiHashMap *map,
+					      const struct GNUNET_HashCode *key,
                                               const void *value)
 {
   union MapEntry me;
@@ -583,7 +729,9 @@ GNUNET_CONTAINER_multihashmap_contains_value (const struct
     struct SmallMapEntry *sme;
 
     for (sme = me.sme; NULL != sme; sme = sme->next)
-      if ( (0 == memcmp (key, sme->key, sizeof (struct GNUNET_HashCode))) &&
+      if ( (0 == memcmp (key,
+			 sme->key,
+			 sizeof (struct GNUNET_HashCode))) &&
 	   (sme->value == value) )
 	return GNUNET_YES;
   }
@@ -592,7 +740,9 @@ GNUNET_CONTAINER_multihashmap_contains_value (const struct
     struct BigMapEntry *bme;
 
     for (bme = me.bme; NULL != bme; bme = bme->next)
-      if ( (0 == memcmp (key, &bme->key, sizeof (struct GNUNET_HashCode))) &&
+      if ( (0 == memcmp (key,
+			 &bme->key,
+			 sizeof (struct GNUNET_HashCode))) &&
 	   (bme->value == value) )
 	return GNUNET_YES;
   }
@@ -613,17 +763,23 @@ grow (struct GNUNET_CONTAINER_MultiHashMap *map)
   unsigned int old_len;
   unsigned int new_len;
   unsigned int idx;
-  unsigned int i;
-
-  map->modification_counter++;
 
   old_map = map->map;
   old_len = map->map_length;
+  GNUNET_assert (0 != old_len);
   new_len = old_len * 2;
-  new_map = GNUNET_malloc (sizeof (union MapEntry) * new_len);
+  if (0 == new_len) /* 2^31 * 2 == 0 */
+    new_len = old_len; /* never use 0 */
+  if (new_len == old_len)
+    return; /* nothing changed */
+  new_map = GNUNET_malloc_large (new_len *
+                                 sizeof (union MapEntry));
+  if (NULL == new_map)
+    return; /* grow not possible */
+  map->modification_counter++;
   map->map_length = new_len;
   map->map = new_map;
-  for (i = 0; i < old_len; i++)
+  for (unsigned int i = 0; i < old_len; i++)
   {
     if (map->use_small_entries)
     {
@@ -748,53 +904,144 @@ GNUNET_CONTAINER_multihashmap_put (struct GNUNET_CONTAINER_MultiHashMap *map,
  *         #GNUNET_SYSERR if it aborted iteration
  */
 int
-GNUNET_CONTAINER_multihashmap_get_multiple (const struct
-                                            GNUNET_CONTAINER_MultiHashMap *map,
+GNUNET_CONTAINER_multihashmap_get_multiple (struct GNUNET_CONTAINER_MultiHashMap *map,
                                             const struct GNUNET_HashCode *key,
                                             GNUNET_CONTAINER_HashMapIterator it,
                                             void *it_cls)
 {
   int count;
-  union MapEntry me;
+  union MapEntry *me;
+  union MapEntry *ce;
 
+  ce = &map->next_cache[map->next_cache_off];
+  GNUNET_assert (++map->next_cache_off < NEXT_CACHE_SIZE);
   count = 0;
-  me = map->map[idx_of (map, key)];
+  me = &map->map[idx_of (map, key)];
   if (map->use_small_entries)
   {
     struct SmallMapEntry *sme;
-    struct SmallMapEntry *nxt;
 
-    nxt = me.sme;
-    while (NULL != (sme = nxt))
+    ce->sme = me->sme;
+    while (NULL != (sme = ce->sme))
     {
-      nxt = sme->next;
-      if (0 != memcmp (key, sme->key, sizeof (struct GNUNET_HashCode)))
+      ce->sme = sme->next;
+      if (0 != memcmp (key,
+		       sme->key,
+		       sizeof (struct GNUNET_HashCode)))
 	continue;
-      if ((it != NULL) && (GNUNET_OK != it (it_cls, key, sme->value)))
+      if ( (NULL != it) &&
+	   (GNUNET_OK != it (it_cls,
+			     key,
+			     sme->value)))
+      {
+	GNUNET_assert (--map->next_cache_off < NEXT_CACHE_SIZE);
 	return GNUNET_SYSERR;
+      }
       count++;
     }
   }
   else
   {
     struct BigMapEntry *bme;
-    struct BigMapEntry *nxt;
 
-    nxt = me.bme;
-    while (NULL != (bme = nxt))
+    ce->bme = me->bme;
+    while (NULL != (bme = ce->bme))
     {
-      nxt = bme->next;
-      if (0 != memcmp (key, &bme->key, sizeof (struct GNUNET_HashCode)))
+      ce->bme = bme->next;
+      if (0 != memcmp (key,
+		       &bme->key,
+		       sizeof (struct GNUNET_HashCode)))
 	continue;
-      if ((it != NULL) && (GNUNET_OK != it (it_cls, key, bme->value)))
+      if ( (NULL != it) &&
+	   (GNUNET_OK != it (it_cls,
+			     key,
+			     bme->value)))
+      {
+	GNUNET_assert (--map->next_cache_off < NEXT_CACHE_SIZE);
 	return GNUNET_SYSERR;
+      }
       count++;
     }
   }
+  GNUNET_assert (--map->next_cache_off < NEXT_CACHE_SIZE);
   return count;
 }
 
 
+/**
+ * @ingroup hashmap
+ * Call @a it on a random value from the map, or not at all
+ * if the map is empty. Note that this function has linear
+ * complexity (in the size of the map).
+ *
+ * @param map the map
+ * @param it function to call on a random entry
+ * @param it_cls extra argument to @a it
+ * @return the number of key value pairs processed, zero or one.
+ */
+unsigned int
+GNUNET_CONTAINER_multihashmap_get_random (const struct GNUNET_CONTAINER_MultiHashMap *map,
+                                          GNUNET_CONTAINER_HashMapIterator it,
+                                          void *it_cls)
+{
+  unsigned int off;
+  unsigned int idx;
+  union MapEntry me;
+
+  if (0 == map->size)
+    return 0;
+  if (NULL == it)
+    return 1;
+  off = GNUNET_CRYPTO_random_u32 (GNUNET_CRYPTO_QUALITY_NONCE,
+                                  map->size);
+  for (idx = 0; idx < map->map_length; idx++)
+  {
+    me = map->map[idx];
+    if (map->use_small_entries)
+    {
+      struct SmallMapEntry *sme;
+      struct SmallMapEntry *nxt;
+
+      nxt = me.sme;
+      while (NULL != (sme = nxt))
+      {
+        nxt = sme->next;
+        if (0 == off)
+        {
+          if (GNUNET_OK != it (it_cls,
+                               sme->key,
+                               sme->value))
+            return GNUNET_SYSERR;
+          return 1;
+        }
+        off--;
+      }
+    }
+    else
+    {
+      struct BigMapEntry *bme;
+      struct BigMapEntry *nxt;
+
+      nxt = me.bme;
+      while (NULL != (bme = nxt))
+      {
+        nxt = bme->next;
+        if (0 == off)
+        {
+          if (GNUNET_OK != it (it_cls,
+                               &bme->key, bme->value))
+            return GNUNET_SYSERR;
+          return 1;
+        }
+        off--;
+      }
+    }
+  }
+  GNUNET_break (0);
+  return GNUNET_SYSERR;
+}
+
+
 /**
  * Create an iterator for a multihashmap.
  * The iterator can be used to retrieve all the elements in the multihashmap
